/***
*
*	Copyright (c) 1999, Valve LLC. All rights reserved.
*	
*	This product contains software technology licensed from Id 
*	Software, Inc. ("Id Technology").  Id Technology (c) 1996 Id Software, Inc. 
*	All Rights Reserved.
*
*   Use, distribution, and modification of this source code and/or resulting
*   object code is restricted to non-commercial enhancements to products from
*   Valve LLC.  All other use, distribution, or modification is prohibited
*   without written permission from Valve LLC.
*
****/
//			
//  hud.h
//
// class CHud declaration
//
// CHud handles the message, calculation, and drawing the HUD
//


#define RGB_YELLOWISH	0x00FFA000 //255,160,0
#define RGB_REDISH		0x00FF1010 //255,160,0
#define RGB_GREENISH	0x0000A000 //0,160,0

#include "wrect.h"
#include "cl_dll.h"
#include "ammo.h"

#include "const.h"
#include "entity_state.h"
#include "cl_entity.h"

#define DHN_DRAWZERO 1
#define DHN_2DIGITS  2
#define DHN_3DIGITS  4
#define MIN_ALPHA	 100	

#define		HUDELEM_ACTIVE	1

typedef struct {
	int x, y;
} POSITION;

typedef struct {
	unsigned char r,g,b,a;
} RGBA;


#define HUD_ACTIVE	1
#define HUD_INTERMISSION 2

#define MAX_PLAYER_NAME_LENGTH		32

#define	MAX_MOTD_LENGTH				1024

//
//-----------------------------------------------------
//
class CHudBase
{
public:
	POSITION  m_pos;
	int   m_type;
	int	  m_iFlags; // active, moving, 
	virtual int Init( void ) {return 0;}
	virtual int VidInit( void ) {return 0;}
	virtual int Draw(float flTime) {return 0;}
	virtual void Think(void) {return;}
	virtual void Reset(void) {return;}
	virtual void InitHUDData( void ) {}		// called every time a server is connected to

};

struct HUDLIST {
	CHudBase	*p;
	HUDLIST		*pNext;
};

typedef struct
{
	float fExpire;
	float fBaseline;
	int	x, y;
} DAMAGE_IMAGE;

class CHudDommage: public CHudBase
{
public:
	virtual int Init( void );
	virtual int Draw(float fTime);
	int MsgFunc_Damage(const char *pszName,  int iSize, void *pbuf);
	int m_HUD_cross;
	void GetPainColor( int &r, int &g, int &b );
	float m_fFade;
	float fGvTemps;
private:
	int iGvbitsDamage;
	int	m_bitsDamage;
};

// WormsHL - 10/07/02 - MP3
class CHudMp3 : public CHudBase
{
public:
	~CHudMp3();
	int Init( void );
	int VidInit( void );
	
private:
	int Draw( float flTime );
	bool deja_init;
	bool no_music;
	bool en_route;
	char chemin_mp3[1024];
};

// WormsHL - 21042002 - Prise d'item
class CHudPrise : public CHudBase
{
public:
	int Init( void );
	int VidInit( void );
	int MsgFunc_Prise (const char *pszName, int iSize, void *pbuf );

private:
	int Draw( float flTime );

	int QuelleItem;
	float Time;
};

// EurOPiuM - 21042002 - Bille Bocket
class CHudBilleBocket : public CHudBase
{
public:
	int MsgFunc_BilleBocket( const char *pszName, int iSize, void *pbuf );
	int Init( void );
	int VidInit( void );

private:
	int Draw( float flTime );

	int ThePuissance;
	int TheIndex;
	// modif - 22/06/02
	int TheIndexAttaquant;

	struct
	{
		char Nom[30];
		// modif - 22/06/02
		char Attaquant[30];
		int EquipeAttaquant;
		float Quand;
		int Force;
		int Equipe;
	}BilleBocket[3];

	int SpriteBilleBocket;
};


// EurOPiuM - 27/06/02 - Radar
class CHudRadar : public CHudBase
{
public:
	int MsgFunc_Radar( const char *pszName, int iSize, void *pbuf );
	int Init( void );
	int VidInit( void );
private:
	int Draw( float flTime );
	struct
	{	
		int type;
		vec_t orgx,orgy;
	}Objets[RADAR_MAXOBJETS+1];
	int max_vision;
};


// EurOPiuM - 16/06/02 - Profil
#include "profil.h"
class CHudProfil : public CHudBase
{
public:
	int MsgFunc_Profil( const char *pszName, int iSize, void *pbuf );
	int Init( void );
	int VidInit( void );

private:
	int Draw( float flTime );

	int TheTeam;
	profil_t TheProfil;
};

//EurOPiuM - 24/07/02 - Particles
// nouveau HUD
class CHudParticles : public CHudBase
{
public:
	int MsgFunc_Lanceflamme( const char *pszName, int iSize, void *pbuf );
	int MsgFunc_Flamme( const char *pszName, int iSize, void *pbuf );
	int MsgFunc_Sainte( const char *pszName, int iSize, void *pbuf );
	int MsgFunc_Explosion( const char *pszName, int iSize, void *pbuf );
	int Init( void );
	int VidInit( void );
};



//-----------------------------
//WormsHL - victoire (EurOPiuM)
//-----------------------------
class CHudVictoire : public CHudBase
{
public:
	int Init( void );
	int VidInit( void );
	int Draw( float flTime );
	int MsgFunc_Victoire (const char *pszName, int iSize, void *pbuf );

private:
	void GfHudHelp( void );
	int TheTeam;
	int Team;
	float Time;
};
//-----------------------------
//WormsHL - victoire (EurOPiuM)
//-----------------------------


//--------------------------
//WormsHL - reste (EurOPiuM)
//--------------------------
class CHudReste : public CHudBase
{
public:
int MsgFunc_Reste( const char *pszName, int iSize, void *pbuf );
int Init( void );

int Draw( float flTime );

int TheReste;
};
//--------------------------
//WormsHL - reste (EurOPiuM)
//--------------------------



//------------------------------
//WormsHL - puissance (EurOPiuM)
//------------------------------
class CHudPuissance : public CHudBase
{
public:
int MsgFunc_Puissance( const char *pszName, int iSize, void *pbuf );
int Init( void );

int Draw( float flTime );

int ThePuissance;
int TheCouleur;
HSPRITE fond;
};
//------------------------------
//WormsHL - puissance (EurOPiuM)
//------------------------------



// EurOPiuM - 21042002 - Croisement
class CHudCroise : public CHudBase
{
public:
	int MsgFunc_Croise( const char *pszName, int iSize, void *pbuf );
	int Init( void );

private:
	int Draw( float flTime );

	int TheIndex;
};

//--------------------------
//WormsHL - score (EurOPiuM)
//--------------------------
class CHudScore : public CHudBase
{
public:

int Init( void );
int Draw( float flTime );

int MsgFunc_Score( const char *pszName, int iSize, void *pbuf );
int iGvScoreBleuPoint, iGvScoreRougePoint, iGvScoreBleuManche, iGvScoreRougeManche;
};
//--------------------------
//WormsHL - score (EurOPiuM)
//--------------------------


//-----------------------------
//WormsHL - compteur (EurOPiuM)
//-----------------------------
class CHudTime : public CHudBase
{
public:

int Init( void );
int Draw( float flTime );

int MsgFunc_Time( const char *pszName, int iSize, void *pbuf );
int TheTime,TheTeam,TheBloque,move,time,maxtime;
int oldTheTeam;
float now,debut;
HSPRITE Fond1,Fond2,Fond3;
HSPRITE FeuR,FeuO,FeuV;
};
//-----------------------------
//WormsHL - compteur (EurOPiuM)
//-----------------------------


class CHudAmmo: public CHudBase
{
public:
	int Init( void );
	int VidInit( void );
	int Draw(float flTime);
	void Think(void);
	void Reset(void);
	int DrawWList(float flTime);
	int MsgFunc_CurWeapon(const char *pszName, int iSize, void *pbuf);
	int MsgFunc_WeaponList(const char *pszName, int iSize, void *pbuf);
	int MsgFunc_AmmoX(const char *pszName, int iSize, void *pbuf);
	int MsgFunc_AmmoPickup( const char *pszName, int iSize, void *pbuf );
	// Groove - 07052002 - Pour ajouter une armes au HUD.
//	int MsgFunc_AddWeapon( const char *pszName, int iSize, void *pbuf )
	int MsgFunc_WeapPickup( const char *pszName, int iSize, void *pbuf );
	int MsgFunc_ItemPickup( const char *pszName, int iSize, void *pbuf );
	int MsgFunc_HideWeapon( const char *pszName, int iSize, void *pbuf );

	void SlotInput( int iSlot );
	void _cdecl UserCmd_Slot1( void );
	void _cdecl UserCmd_Slot2( void );
	void _cdecl UserCmd_Slot3( void );
	void _cdecl UserCmd_Slot4( void );
	void _cdecl UserCmd_Slot5( void );
	void _cdecl UserCmd_Slot6( void );
	void _cdecl UserCmd_Slot7( void );
	void _cdecl UserCmd_Slot8( void );
	void _cdecl UserCmd_Slot9( void );
	void _cdecl UserCmd_Slot10( void );
	void _cdecl UserCmd_Close( void );
	void _cdecl UserCmd_NextWeapon( void );
	void _cdecl UserCmd_PrevWeapon( void );

	WEAPON *m_pWeapon;

private:
	float m_fFade;
	RGBA  m_rgba;
	int	m_HUD_bucket0;
	int m_HUD_selection;

};

#include "health.h"


#define FADE_TIME 100


//
//-----------------------------------------------------
//
class CHudGeiger: public CHudBase
{
public:
	int Init( void );
	int VidInit( void );
	int Draw(float flTime);
	int MsgFunc_Geiger(const char *pszName, int iSize, void *pbuf);
	
private:
	int m_iGeigerRange;

};

//
//-----------------------------------------------------
//
class CHudTrain: public CHudBase
{
public:
	int Init( void );
	int VidInit( void );
	int Draw(float flTime);
	int MsgFunc_Train(const char *pszName, int iSize, void *pbuf);

private:
	HSPRITE m_hSprite;
	int m_iPos;

};

//
//-----------------------------------------------------
//
// REMOVED: Vgui has replaced this.
//
/*
class CHudMOTD : public CHudBase
{
public:
	int Init( void );
	int VidInit( void );
	int Draw( float flTime );
	void Reset( void );

	int MsgFunc_MOTD( const char *pszName, int iSize, void *pbuf );

protected:
	static int MOTD_DISPLAY_TIME;
	char m_szMOTD[ MAX_MOTD_LENGTH ];
	float m_flActiveRemaining;
	int m_iLines;
};
*/

//
//-----------------------------------------------------
//
class CHudStatusBar : public CHudBase
{
public:
	int Init( void );
	int VidInit( void );
	int Draw( float flTime );
	void Reset( void );
	void ParseStatusString( int line_num );

	int MsgFunc_StatusText( const char *pszName, int iSize, void *pbuf );
	int MsgFunc_StatusValue( const char *pszName, int iSize, void *pbuf );

protected:
	enum { 
		MAX_STATUSTEXT_LENGTH = 128,
		MAX_STATUSBAR_VALUES = 8,
		MAX_STATUSBAR_LINES = 2,
	};

	char m_szStatusText[MAX_STATUSBAR_LINES][MAX_STATUSTEXT_LENGTH];  // a text string describing how the status bar is to be drawn
	char m_szStatusBar[MAX_STATUSBAR_LINES][MAX_STATUSTEXT_LENGTH];	// the constructed bar that is drawn
	int m_iStatusValues[MAX_STATUSBAR_VALUES];  // an array of values for use in the status bar

	int m_bReparseString; // set to TRUE whenever the m_szStatusBar needs to be recalculated
};

//
//-----------------------------------------------------
//
// REMOVED: Vgui has replaced this.
//
/*
class CHudScoreboard: public CHudBase
{
public:
	int Init( void );
	void InitHUDData( void );
	int VidInit( void );
	int Draw( float flTime );
	int DrawPlayers( int xoffset, float listslot, int nameoffset = 0, char *team = NULL ); // returns the ypos where it finishes drawing
	void UserCmd_ShowScores( void );
	void UserCmd_HideScores( void );
	int MsgFunc_ScoreInfo( const char *pszName, int iSize, void *pbuf );
	int MsgFunc_TeamInfo( const char *pszName, int iSize, void *pbuf );
	int MsgFunc_TeamScore( const char *pszName, int iSize, void *pbuf );
	void DeathMsg( int killer, int victim );

	int m_iNumTeams;

	int m_iLastKilledBy;
	int m_fLastKillTime;
	int m_iPlayerNum;
	int m_iShowscoresHeld;

	void GetAllPlayersInfo( void );
private:
	struct cvar_s *cl_showpacketloss;

};
*/

enum 
{ 
	MAX_PLAYERS = 64,
	MAX_TEAMS = 64,
	MAX_TEAM_NAME = 16,
};

struct extra_player_info_t 
{
	short frags;
	short deaths;
	short playerclass;
	short teamnumber;
	char teamname[MAX_TEAM_NAME];
};

struct team_info_t 
{
	char name[MAX_TEAM_NAME];
	short frags;
	short deaths;
	short ping;
	short packetloss;
	short ownteam;
	short players;
	int already_drawn;
	int scores_overriden;
	int teamnumber;
};

extern hud_player_info_t	g_PlayerInfoList[MAX_PLAYERS+1];	   // player info from the engine
extern extra_player_info_t  g_PlayerExtraInfo[MAX_PLAYERS+1];   // additional player info sent directly to the client dll
extern team_info_t			g_TeamInfo[MAX_TEAMS+1];
extern int					g_IsSpectator[MAX_PLAYERS+1];


//
//-----------------------------------------------------
//
class CHudDeathNotice : public CHudBase
{
public:
	int Init( void );
	void InitHUDData( void );
	int VidInit( void );
	int Draw( float flTime );
	int MsgFunc_DeathMsg( const char *pszName, int iSize, void *pbuf );

private:
	int m_HUD_d_skull;  // sprite index of skull icon
};

//
//-----------------------------------------------------
//
class CHudMenu : public CHudBase
{
public:
	int Init( void );
	void InitHUDData( void );
	int VidInit( void );
	void Reset( void );
	int Draw( float flTime );
	int MsgFunc_ShowMenu( const char *pszName, int iSize, void *pbuf );

	void SelectMenuItem( int menu_item );

	int m_fMenuDisplayed;
	int m_bitsValidSlots;
	float m_flShutoffTime;
	int m_fWaitingForMore;
};

//
//-----------------------------------------------------
//
class CHudSayText : public CHudBase
{
public:
	int Init( void );
	void InitHUDData( void );
	int VidInit( void );
	int Draw( float flTime );
	int MsgFunc_SayText( const char *pszName, int iSize, void *pbuf );
	void SayTextPrint( const char *pszBuf, int iBufSize, int clientIndex = -1 );
	void EnsureTextFitsInOneLineAndWrapIfHaveTo( int line );
};

//
//-----------------------------------------------------
//
/*
class CHudBattery: public CHudBase
{
public:
	int Init( void );
	int VidInit( void );
	int Draw(float flTime);
	int MsgFunc_Battery(const char *pszName,  int iSize, void *pbuf );
	
private:
	HSPRITE m_hSprite1;
	HSPRITE m_hSprite2;
	wrect_t *m_prc1;
	wrect_t *m_prc2;
	int	  m_iBat;	
	float m_fFade;
	int	  m_iHeight;		// width of the battery innards
};
*/

//
//-----------------------------------------------------
//
/*
class CHudFlashlight: public CHudBase
{
public:
	int Init( void );
	int VidInit( void );
	int Draw(float flTime);
	void Reset( void );
	int MsgFunc_Flashlight(const char *pszName,  int iSize, void *pbuf );
	int MsgFunc_FlashBat(const char *pszName,  int iSize, void *pbuf );
	
private:
	HSPRITE m_hSprite1;
	HSPRITE m_hSprite2;
	HSPRITE m_hBeam;
	wrect_t *m_prc1;
	wrect_t *m_prc2;
	wrect_t *m_prcBeam;
	float m_flBat;	
	int	  m_iBat;	
	int	  m_fOn;
	float m_fFade;
	int	  m_iWidth;		// width of the battery innards
};
*/
//
//-----------------------------------------------------
//
const int maxHUDMessages = 16;
struct message_parms_t
{
	client_textmessage_t	*pMessage;
	float	time;
	int x, y;
	int	totalWidth, totalHeight;
	int width;
	int lines;
	int lineLength;
	int length;
	int r, g, b;
	int text;
	int fadeBlend;
	float charTime;
	float fadeTime;
};

//
//-----------------------------------------------------
//

class CHudTextMessage: public CHudBase
{
public:
	int Init( void );
	static char *LocaliseTextString( const char *msg, char *dst_buffer, int buffer_size );
	static char *BufferedLocaliseTextString( const char *msg );
	char *LookupString( const char *msg_name, int *msg_dest = NULL );
	int MsgFunc_TextMsg(const char *pszName, int iSize, void *pbuf);
};

//
//-----------------------------------------------------
//

class CHudMessage: public CHudBase
{
public:
	int Init( void );
	int VidInit( void );
	int Draw(float flTime);
	int MsgFunc_HudText(const char *pszName, int iSize, void *pbuf);
	int MsgFunc_GameTitle(const char *pszName, int iSize, void *pbuf);

	float FadeBlend( float fadein, float fadeout, float hold, float localTime );
	int	XPosition( float x, int width, int lineWidth );
	int YPosition( float y, int height );

	void MessageAdd( const char *pName, float time );
	void MessageDrawScan( client_textmessage_t *pMessage, float time );
	void MessageScanStart( void );
	void MessageScanNextChar( void );
	void Reset( void );

private:
	client_textmessage_t		*m_pMessages[maxHUDMessages];
	float						m_startTime[maxHUDMessages];
	message_parms_t				m_parms;
	float						m_gameTitleTime;
	client_textmessage_t		*m_pGameTitle;

	int m_HUD_title_life;
	int m_HUD_title_half;
};

//
//-----------------------------------------------------
//
#define MAX_SPRITE_NAME_LENGTH	24

class CHudStatusIcons: public CHudBase
{
public:
	int Init( void );
	int VidInit( void );
	void Reset( void );
	int Draw(float flTime);
	int MsgFunc_StatusIcon(const char *pszName, int iSize, void *pbuf);

	enum { 
		MAX_ICONSPRITENAME_LENGTH = MAX_SPRITE_NAME_LENGTH,
		MAX_ICONSPRITES = 4,
	};

	
	//had to make these public so CHud could access them (to enable concussion icon)
	//could use a friend declaration instead...
	void EnableIcon( char *pszIconName, unsigned char red, unsigned char green, unsigned char blue );
	void DisableIcon( char *pszIconName );

private:

	typedef struct
	{
		char szSpriteName[MAX_ICONSPRITENAME_LENGTH];
		HSPRITE spr;
		wrect_t rc;
		unsigned char r, g, b;
	} icon_sprite_t;

	icon_sprite_t m_IconList[MAX_ICONSPRITES];

};

//
//-----------------------------------------------------
//

typedef struct cvar_s cvar_t;

class CHud
{
private:
	HUDLIST						*m_pHudList;
	HSPRITE						m_hsprLogo;
	int							m_iLogo;
	client_sprite_t				*m_pSpriteList;
	int							m_iSpriteCount;
	int							m_iSpriteCountAllRes;
	float						m_flMouseSensitivity;
	int							m_iConcussionEffect; 

public:

	HSPRITE						m_hsprCursor;
	float m_flTime;	   // the current client time
	float m_fOldTime;  // the time at which the HUD was last redrawn
	double m_flTimeDelta; // the difference between flTime and fOldTime
	Vector	m_vecOrigin;
	Vector	m_vecAngles;
	int		m_iKeyBits;
	int		m_iHideHUDDisplay;
	int		m_iFOV;
	int		m_Teamplay;
	int		m_iRes;
	cvar_t  *m_pCvarStealMouse;

	//EurOPiUM - enlever/remettre pointeur - 19/07/05
	WEAPON * ArmeSave; // arme courante
	void enlever_crosshair();
	void remettre_crosshair();


	// EurOPiuM - 18032002 - Profil
	short IsBot[MAX_PLAYERS];
	// EurOPiUM - 20/06/02 - Particles engine
	bool affiche_lanceflamme;
	// EurOPiuM - 21/06/02 - nouvelle icone arme
	int arme_ramassee;

	int m_iFontHeight;
	int DrawHudNumber(int x, int y, int iFlags, int iNumber, int r, int g, int b );
//WormsHL - chiffres en couleur (EurOPiuM)
	int DrawHudNumberBlue(int x, int y, int iFlags, int iNumber, int r, int g, int b );
	int DrawHudNumberRed(int x, int y, int iFlags, int iNumber, int r, int g, int b );

	int DrawHudString(int x, int y, int iMaxX, char *szString, int r, int g, int b );
	int DrawHudStringReverse( int xpos, int ypos, int iMinX, char *szString, int r, int g, int b );
	int DrawHudNumberString( int xpos, int ypos, int iMinX, int iNumber, int r, int g, int b );
	int GetNumWidth(int iNumber, int iFlags);

private:
	// the memory for these arrays are allocated in the first call to CHud::VidInit(), when the hud.txt and associated sprites are loaded.
	// freed in ~CHud()
	HSPRITE *m_rghSprites;	/*[HUD_SPRITE_COUNT]*/			// the sprites loaded from hud.txt
	wrect_t *m_rgrcRects;	/*[HUD_SPRITE_COUNT]*/
	char *m_rgszSpriteNames; /*[HUD_SPRITE_COUNT][MAX_SPRITE_NAME_LENGTH]*/

	struct cvar_s *default_fov;
public:
	HSPRITE GetSprite( int index ) 
	{
		return (index < 0) ? 0 : m_rghSprites[index];
	}

	wrect_t& GetSpriteRect( int index )
	{
		return m_rgrcRects[index];
	}

	int GetSpriteIndex( const char *SpriteName );	// gets a sprite index, for use in the m_rghSprites[] array

	//WormsHL - mp3 (EurOPiuM)
	CHudMp3				m_Mp3;
	//WormsHL - compteur (EurOPiuM)
	CHudTime			m_Time;
	//EurOPiuM - 21042002 - Recupration des items
	CHudPrise			m_Prise;
	//WormsHL - croise (EurOPiuM)
	CHudCroise			m_Croise;
	//WormsHL - victoire (EurOPiuM)
	CHudVictoire		m_Victoire;
	//WormsHL - score (EurOPiuM)
	CHudScore			m_Score;
	//WormsHL - reste (EurOPiuM)
	CHudReste			m_Reste;
	//WormsHL - puissance (EurOPiuM)
	CHudPuissance		m_Puissance;
	//Groove - 26022002 - Indicateur d'armes slectionner
//	CHudArme m_Arme;
	// Groove - 22042002 - Indicateur pour le bille bocket
	CHudBilleBocket		m_BilleBocket;
	CHudAmmo			m_Ammo;
	CHudHealth			m_Health;
	CHudDommage			m_Dommage;
	CHudGeiger			m_Geiger;
	CHudTrain			m_Train;
	CHudMessage			m_Message;
	CHudStatusBar		m_StatusBar;
	CHudDeathNotice		m_DeathNotice;
	CHudSayText			m_SayText;
	CHudMenu			m_Menu;
	CHudTextMessage		m_TextMessage;
	CHudStatusIcons		m_StatusIcons;
	// EurOPiuM - 16/06/02 - Profil
	CHudProfil			m_Profil;

//EurOPiuM - 24/07/02 - Particles
// plus besoin des ces hud -> on les vire !
	// EurOPiuM - 26/06/02 - Particles Lanceflamme
/*	CHudLanceflamme m_Lanceflamme;
	// EurOPiuM - 26/06/02 - Particles Flamme
	CHudFlamme m_Flamme;
*/
	
	//EurOPiuM - 24/07/02 - Particles
	// nouveau HUD
	CHudParticles m_Particles;
	// EurOPiuM - 27/06/02 - Radar
	// bien le laisser en dernier pour qu'il apparaisse au dessus de tous les autres sprites
	CHudRadar m_Radar;

	void Init( void );
	void VidInit( void );
	void Think(void);
	int Redraw( float flTime, int intermission );
	int UpdateClientData( client_data_t *cdata, float time );

	CHud() : m_iSpriteCount(0), m_pHudList(NULL) {}  
	~CHud();			// destructor, frees allocated memory

	// user messages
	int _cdecl MsgFunc_Damage( const char *pszName, int iSize, void *pbuf );
	int _cdecl MsgFunc_GameMode( const char *pszName, int iSize, void *pbuf );
	int _cdecl MsgFunc_Logo( const char *pszName,  int iSize, void *pbuf);
	int _cdecl MsgFunc_ResetHUD( const char *pszName,  int iSize, void *pbuf);
	void _cdecl MsgFunc_InitHUD( const char *pszName, int iSize, void *pbuf );
	int _cdecl MsgFunc_SetFOV( const char *pszName,  int iSize, void *pbuf);
	int _cdecl MsgFunc_Concuss( const char *pszName, int iSize, void *pbuf );
	// Screen information
	SCREENINFO	m_scrinfo;

//	int	m_iWeaponBits;
	// Groove - 08052002 - Gestion des armes de WormsHL
	// __int64 iGvArmes;
	short iGvArmes[MAX_WEAPON_SLOTS];

	int	m_fPlayerDead;
	int m_iIntermission;

	// sprite indexes

	// WormsHL - chiffres en couleurs (EurOPiuM)
	int m_HUD_number_0;

	void AddHudElem(CHudBase *p);

	float GetSensitivity();

// EurOPiuM - 21/04/2002 - Variables pour recuperer des valeurs du delta.lst
	cl_entity_t *localplayer;
};


class TeamFortressViewport;

extern CHud gHUD;
extern TeamFortressViewport *gViewPort;

extern int g_iPlayerClass;
extern int g_iTeamNumber;
extern int g_iUser1;
extern int g_iUser2;
