/*******************************\
*	WormsHL						*
*		www.wormshl.fr.st		*
*								*
*	Groove						*
*		groove@wormshl.net		*
*								*
*	arme_air_strike.cpp			*
*		19022002				*
\*******************************/

#include "extdll.h"
#include "util.h"
#include "cbase.h"
#include "player.h"
#include "fx_sprite.h"
#include "generic_arme.h"
#include "arme_air_strike.h"
#include "game.h"
#include "gamerules.h"

#ifdef Gd_ARME_AIR_STRIKE

LINK_ENTITY_TO_CLASS( weapon_air_strike, GcAirStrike );
LINK_ENTITY_TO_CLASS( rocket_air_strike, GcAirStrikeRocket );

int GcAirStrike::GetItemInfo( ItemInfo *p )
{
	p->pszName = STRING( pev->classname );
	p->pszAmmo1 = "munition_air_strike";
	p->iMaxAmmo1 = Gm_MUNI_MAX;
	p->iSlot = Gm_AIR_STRIKE_SLOT;
	p->iPosition = Gm_AIR_STRIKE_POSITION;
	p->iId = m_iId = Gm_AIR_STRIKE_ID;
	p->iWeight = Gm_AIR_STRIKE_POID;
	p->iFlags = ITEM_FLAG_LIMITINWORLD | ITEM_FLAG_EXHAUSTIBLE;
	return 1;
}

void GcAirStrike::Spawn( void )
{
	Precache( );

	m_iId = Gm_AIR_STRIKE_ID;
	iGvAmmoDefault = Gm_AIR_STRIKE_MUNITION;
	
	FallInit( );
}

void GcAirStrike::Precache( void )
{
	PRECACHE_MODEL( Gm_AIR_STRIKE_FICHIER_MODEL_R );
	PRECACHE_MODEL( Gm_AIR_STRIKE_FICHIER_MODEL_V );
	PRECACHE_MODEL( Gm_AIR_STRIKE_FICHIER_MODEL_P );
	PRECACHE_MODEL( Gm_AIR_STRIKE_FICHIER_SPRITE_CIBLE );

	PRECACHE_SOUND( Gm_AIR_STRIKE_FICHIER_SON_TRAINE );
	PRECACHE_SOUND( Gm_AIR_STRIKE_FICHIER_SON_CIBLE );
}

BOOL GcAirStrike::Deploy( void )
{
	GvTempsAnim_f = gpGlobals->time + 2.0;
	return DefaultDeploy( Gm_AIR_STRIKE_FICHIER_MODEL_V, Gm_AIR_STRIKE_FICHIER_MODEL_P, Ga_AIR_STRIKE_DRAW, "air_strike" );
}

void GcAirStrike::Holster( void )
{
	if( m_pSpot )
	{
		m_pSpot->Killed( NULL, GIB_NEVER );
		m_pSpot = NULL;
	}

	if( !m_pPlayer->m_rgAmmo[iGvAmmoID] )
	{
		m_pPlayer->iGvArmes[Gm_AIR_STRIKE_SLOT] &= ~( 1<<Gm_AIR_STRIKE_POSITION );
		SetThink( DestroyItem );
		pev->nextthink = gpGlobals->time + 0.1;
	}

	GfFinDeTour( &GvTempsFDT_f, 0 );
}

void GcAirStrike::GfAttack1( void )
{
	if( !m_pPlayer->GvPlayerTPT.bTire || !m_pPlayer->m_rgAmmo[iGvAmmoID] )
	{
		GfAttack0( );
		return;
	}

	if( GvChoixTire_i == 2 )
	{
		GvChoixTire_i = 1;
		GfTire2( );
	}
	else
	{
		GvChoixTire_i = 2;
		GfTire1( );
	}

	fGvAttack0 = fGvAttack1 = gpGlobals->time + 3.0;
}

void GcAirStrike::GfTire1( void )
{
	// Groove - 15032002 - Animations
	SendWeaponAnim( Ga_AIR_STRIKE_ATTACK_PREPA );
	GvTempsAnim_f = gpGlobals->time + 3.0;
	m_pPlayer->SetAnimation( PLAYER_ATTACK1 );

	// Groove - 15032002 - Positions du pointeur laser
	GvPosition_c = UpdateSpot( );

	m_pSprite = CSprite::SpriteCreate( Gm_AIR_STRIKE_FICHIER_SPRITE_CIBLE, GvPosition_c, FALSE );
	m_pSprite->pev->scale = 2.0;
	m_pSprite->SetTransparency( kRenderGlow, 255, 255, 255, 255,kRenderFxNoDissipation );
	
	EMIT_SOUND( ENT( m_pPlayer->pev ), CHAN_WEAPON, Gm_AIR_STRIKE_FICHIER_SON_CIBLE, 0.7, ATTN_NORM );
}

void GcAirStrike::GfTire2( void )
{
	// Groove - 15032002 - Animations
	SendWeaponAnim( Ga_AIR_STRIKE_ATTACK );
	GvTempsAnim_f = gpGlobals->time + 2.0;
	m_pPlayer->SetAnimation( PLAYER_ATTACK1 );

	if( m_pSprite )
	{
		UTIL_Remove( m_pSprite );
		m_pSprite = NULL;
	}

	// Groove - 15032002 - Positions du depart de l'eclair
	Vector GvOrigin_c = GvPosition_c + Vector( 0,0, 1000 );

	for( short I = 10; I >= 1; I-- )
		GcAirStrikeRocket::CreateRocket( m_pPlayer, GvOrigin_c + Vector( RANDOM_LONG( -200, 200 ),RANDOM_LONG( -200, 200),0 ), Vector( 0,0,-1 ), GfDommage( Gm_AIR_STRIKE_DOMMAGE ));

	EMIT_SOUND( ENT( m_pPlayer->pev ), CHAN_WEAPON, Gm_AIR_STRIKE_FICHIER_SON_TRAINE, 0.9, ATTN_NORM );

	m_pPlayer->m_rgAmmo[iGvAmmoID]--;

	m_pPlayer->GvPlayerTPT.bAmmo = true;
	m_pPlayer->GvPlayerTPT.bTire = false;
	GvTempsFDT_f = gpGlobals->time;
}

void GcAirStrike::GfAttack2( void )
{
	GfAttack0( );
	fGvAttack2 = gpGlobals->time + 0.5;
}

void GcAirStrike::GfAttack0( void )
{
	fGvAttack0 = gpGlobals->time + 1.0;
	GfFinDeTour( &GvTempsFDT_f, Gm_AIR_STRIKE_TEMPS_DEPLACEMENT );

	UpdateSpot( );

	if( !m_pPlayer->GvPlayerTPT.bTire )
		GvChoixTire_i = 1;

	if(( m_pSprite )&&( !m_pPlayer->GvPlayerTPT.bTire ))
	{
		UTIL_Remove( m_pSprite );
		m_pSprite = NULL;
	}

	if( GvTempsAnim_f &&( GvTempsAnim_f < gpGlobals->time ))
	{
		if( m_pPlayer->m_rgAmmo[iGvAmmoID] )
		{
			int GvAnim_i;
			if( GvChoixTire_i == 2 )
			{
				GvAnim_i = Ga_AIR_STRIKE_ATTACK_ILDE;
				GvTempsAnim_f = gpGlobals->time + 2.0;
			}
			else
			{
				switch( RANDOM_LONG( 1,2 ))
				{
				default:
				case 1:
					GvAnim_i = Ga_AIR_STRIKE_IDLE1;
					GvTempsAnim_f = gpGlobals->time + 3.0;
					break;
				case 2:
					GvAnim_i = Ga_AIR_STRIKE_IDLE2;
					GvTempsAnim_f = gpGlobals->time + 3.0;
					break;
				}
			}
			SendWeaponAnim( GvAnim_i );
		}
		else
		{
			m_pPlayer->pev->viewmodel = 0; 
			m_pPlayer->pev->weaponmodel = 0;
		}
	}
}

////////////////////////////////////////////////////////

GcAirStrikeRocket *GcAirStrikeRocket::CreateRocket( CBasePlayer *pGvPlayer, Vector GvOrigin, Vector GvAngles, int iGvDommage )
{
	GcAirStrikeRocket *pRocket = GetClassPtr(( GcAirStrikeRocket *)NULL );

	pRocket->pGvPlayer = pGvPlayer;
	if( pRocket->pGvPlayer )
		pRocket->pev->owner = ENT( pGvPlayer->pev );

	UTIL_SetOrigin( pRocket->pev, GvOrigin );
	pRocket->pev->angles = GvAngles;
	pRocket->pev->dmg = iGvDommage;
	pRocket->Spawn( );

	return pRocket;
}

void GcAirStrikeRocket :: GfActualise( void )
{
	pev->nextthink = gpGlobals->time + 0.05;
	pev->velocity += g_pGameRules->GvVent;
	pev->frame = ( static_cast<int>( pev->frame + 10 )) % 255;
//	pev->angles = UTIL_VecToAngles( pev->velocity );
}

void GcAirStrikeRocket :: GfTraine( void )
{
	MESSAGE_BEGIN( MSG_BROADCAST, SVC_TEMPENTITY );
		WRITE_BYTE( TE_BEAMFOLLOW );
		WRITE_SHORT( entindex( ));	// entity
		WRITE_SHORT( GvTrail_i );	// model
		WRITE_BYTE( 40 );			// life
		WRITE_BYTE( 8  );			// width
		WRITE_BYTE( 64 );			// r
		WRITE_BYTE( 192 );			// g
		WRITE_BYTE( 255 );			// b
		WRITE_BYTE( 255 );			// brightness
	MESSAGE_END( );  // move PHS/PVS data sending into here (SEND_ALL, SEND_PVS, SEND_PHS)
}

void GcAirStrikeRocket :: Spawn( void )
{
	Precache( );

	pev->movetype = MOVETYPE_BOUNCE;
	pev->solid = SOLID_BBOX;

	SET_MODEL( ENT( pev ), Gm_AIR_STRIKE_FICHIER_MODEL_R );
	UTIL_SetSize( pev, Vector( -2,-2,-2 ), Vector( 2,2,2 ));
	UTIL_SetOrigin( pev, pev->origin );

	pev->classname = MAKE_STRING( "air_strike" );

	if( GvQTrailCMD_i )
		GfTraine( );

	SetTouch( GfTouch );
	SetThink( GfActualise );

	pev->angles = Vector( 0,0,0 );
	pev->velocity.z -= RANDOM_LONG( 0,32 );
	pev->gravity = RANDOM_FLOAT( 0.6,0.8 );

	pev->nextthink = gpGlobals->time;
}

void GcAirStrikeRocket :: GfTouch( CBaseEntity *pOther )
{
	pGvPlayer->GvPlayerTPT.bAmmo = false;
/*
	MESSAGE_BEGIN( MSG_ALL, gmsgExplosion);
		WRITE_BYTE( PARTICLES_EXPLOSION_PUISSANCE_450 ); //puissance
		WRITE_BYTE( PARTICLES_EXPLOSION_COULEUR_BLEU ); //couleur
		WRITE_COORD( pev->origin.x ); // coordonne x
		WRITE_COORD( pev->origin.y ); // coordonne y
		WRITE_COORD( pev->origin.z ); // coordonne z
	MESSAGE_END( );*/

	GfGeneralDetonationTouch( pOther );
}

void GcAirStrikeRocket :: Precache( void )
{
	GvTrail_i = PRECACHE_MODEL( Gm_AIR_STRIKE_FICHIER_SPRITE_TRAINE );
}

#endif
