/*******************************\
*	WormsHL						*
*		www.wormshl.fr.st		*
*								*
*	Groove						*
*		groove@wormshl.net		*
*								*
*	arme_hache.cpp				*
*		19022002				*
\*******************************/

#include "extdll.h"
#include "util.h"
#include "cbase.h"
#include "player.h"
#include "generic_arme.h"
#include "arme_hache.h"
#include "gamerules.h"

#ifdef Gd_ARME_HACHE

LINK_ENTITY_TO_CLASS( weapon_hache, GcHache );

int GcHache::GetItemInfo( ItemInfo *p )
{
	p->pszName = STRING(pev->classname);
	p->pszAmmo1 = "ammo_hache";
	p->iMaxAmmo1 = Gm_MUNI_MAX;
	p->iSlot = Gm_HACHE_SLOT;
	p->iPosition = Gm_HACHE_POSITION;
	p->iId = Gm_HACHE_ID;
	p->iWeight = Gm_HACHE_POID;
	p->iFlags = ITEM_FLAG_LIMITINWORLD | ITEM_FLAG_EXHAUSTIBLE;
	return 1;
}

void GcHache::Spawn( void )
{
	Precache( );

	m_iId = Gm_HACHE_ID;
	iGvAmmoDefault = Gm_HACHE_MUNITION;

	GvTempsFDT_f = 0;

	FallInit( );
}

void GcHache::Precache( void )
{
	PRECACHE_MODEL( Gm_HACHE_FICHIER_MODEL_V );
	PRECACHE_MODEL( Gm_HACHE_FICHIER_MODEL_P );

	PRECACHE_SOUND( Gm_HACHE_FICHIER_SON_HIT_01 );
	PRECACHE_SOUND( Gm_HACHE_FICHIER_SON_HIT_02 );
	PRECACHE_SOUND( Gm_HACHE_FICHIER_SON_HIT_CORPS_01 );
	PRECACHE_SOUND( Gm_HACHE_FICHIER_SON_MANQUER );
	PRECACHE_SOUND( Gm_HACHE_FICHIER_SON_DEDLOY );
}

BOOL GcHache::Deploy( void )
{
	EMIT_SOUND( ENT( m_pPlayer->pev ), CHAN_WEAPON, Gm_HACHE_FICHIER_SON_DEDLOY, 1, ATTN_NORM);
	GvTempsAnim_f = gpGlobals->time + 2.0;
	return DefaultDeploy( Gm_HACHE_FICHIER_MODEL_V, Gm_HACHE_FICHIER_MODEL_P, Ga_HACHE_DRAW, "hache" );
}

void GcHache::Holster( void )
{
	if( m_pSpot )
	{
		m_pSpot->Killed( NULL, GIB_NEVER );
		m_pSpot = NULL;
	}

	if( !m_pPlayer->m_rgAmmo[iGvAmmoID] )
	{
		m_pPlayer->iGvArmes[Gm_HACHE_SLOT] &= ~( 1<<Gm_HACHE_POSITION );
		SetThink( DestroyItem );
		pev->nextthink = gpGlobals->time + 0.1;
	}

	GfFinDeTour( &GvTempsFDT_f, 0 );
}

void GcHache::GfAttack1( void )
{
	if( !m_pPlayer->GvPlayerTPT.bTire || !m_pPlayer->m_rgAmmo[iGvAmmoID] )
	{
		GfAttack0( );
		return;
	}

	GvTire( );
}

void GcHache::GvTire( void )
{
	TraceResult tr;

	UTIL_MakeVectors( m_pPlayer->pev->v_angle );
	Vector GvOrigin_c	= m_pPlayer->GetGunPosition( );
	Vector GvFin_c = GvOrigin_c + gpGlobals->v_forward * 32;

	UTIL_TraceLine( GvOrigin_c, GvFin_c, dont_ignore_monsters, ENT( m_pPlayer->pev ), &tr );

	if ( tr.flFraction >= 1.0 )
	{
		UTIL_TraceHull( GvOrigin_c, GvFin_c, dont_ignore_monsters, head_hull, ENT( m_pPlayer->pev ), &tr );
		if ( tr.flFraction < 1.0 )
		{
			// Calculate the point of intersection of the line (or hull) and the object we hit
			// This is and approximation of the "best" intersection
			CBaseEntity *pHit = CBaseEntity::Instance( tr.pHit );
			if ( !pHit || pHit->IsBSPModel() )
				FindHullIntersection( GvFin_c, tr, VEC_DUCK_HULL_MIN, VEC_DUCK_HULL_MAX, m_pPlayer->edict() );
			GvFin_c = tr.vecEndPos;	// This is the point on the actual surface (the hull could have hit space)
		}
	}

	m_pPlayer->SetAnimation( PLAYER_ATTACK1 );
	if ( tr.flFraction >= 1.0 )
	{
		SendWeaponAnim( Ga_HACHE_ATTACK );
		GvTempsAnim_f = gpGlobals->time + 2;
		fGvAttack1 = gpGlobals->time + 0.5;
		// play wiff or swish sound
		EMIT_SOUND_DYN( ENT( m_pPlayer->pev ), CHAN_WEAPON, Gm_HACHE_FICHIER_SON_MANQUER, 1, ATTN_NORM, 0, 94 + RANDOM_LONG(0,0xF));
	}
	else
	{
		GfSonAttack( );
		SendWeaponAnim( Ga_HACHE_ATTACK );

		CBaseEntity *pEntity = CBaseEntity::Instance( tr.pHit );

		ClearMultiDamage( );
		pEntity->TakeDamage ( pev, pev, pEntity->pev->health / 2, Gm_DMG_CONTACT );
		if( pEntity->pev->health <= 1 )
			pEntity->TakeDamage ( pev, pev, 25, Gm_DMG_CONTACT );

		ApplyMultiDamage( m_pPlayer->pev, m_pPlayer->pev );

		int GvHitWorld_b = true;

		if( pEntity )
		{
			if( pEntity->Classify( ) != CLASS_NONE && pEntity->Classify( ) != CLASS_MACHINE )
			{
				EMIT_SOUND( ENT( m_pPlayer->pev ), CHAN_WEAPON, Gm_HACHE_FICHIER_SON_HIT_CORPS_01, 1, ATTN_NORM);
				GvHitWorld_b = false;
			}
		}

		if( GvHitWorld_b )
		{
			// Sons de l'attaque
			switch( RANDOM_LONG( 0,1 ))
			{
			case 0:
				EMIT_SOUND_DYN(ENT(m_pPlayer->pev), CHAN_ITEM, Gm_HACHE_FICHIER_SON_HIT_01, 1.0, ATTN_NORM, 0, 98 + RANDOM_LONG(0,3)); 
				break;
			case 1:
				EMIT_SOUND_DYN(ENT(m_pPlayer->pev), CHAN_ITEM, Gm_HACHE_FICHIER_SON_HIT_02, 1.0, ATTN_NORM, 0, 98 + RANDOM_LONG(0,3)); 
				break;
			}
		}
		else
		{
			EMIT_SOUND( ENT( m_pPlayer->pev ), CHAN_WEAPON, Gm_HACHE_FICHIER_SON_HIT_CORPS_01, 1, ATTN_NORM);
		}
	}

	m_pPlayer->m_rgAmmo[iGvAmmoID]--;

	GvTempsAnim_f = gpGlobals->time + 1.5;

	m_pPlayer->GvPlayerTPT.bTire = false;
	GvTempsFDT_f = gpGlobals->time;
}

void GcHache::GfAttack2( void )
{
	GfAttack0( );
	fGvAttack2 = gpGlobals->time + 0.5;
}

void GcHache::GfAttack0( void )
{
	fGvAttack0 = gpGlobals->time + 1.0;
	GfFinDeTour( &GvTempsFDT_f, Gm_HACHE_TEMPS_DEPLACEMENT );

	if( m_pPlayer->iGvPlayerOutil & Gm_PLAYER_OUTIL_VISEE )
		UpdateSpot( );
	else if( m_pSpot &&!( m_pPlayer->iGvPlayerOutil & Gm_PLAYER_OUTIL_VISEE ))
	{
		m_pSpot->Killed( NULL, GIB_NEVER );
		m_pSpot = NULL;
	}

	if( GvTempsAnim_f &&( GvTempsAnim_f < gpGlobals->time ))
	{
		if( m_pPlayer->m_rgAmmo[iGvAmmoID] )
		{
			short GvAnim_i;
			switch( RANDOM_LONG(1,3))
			{
			default:
			case 1:
				GvAnim_i = Ga_HACHE_IDLE_01;
				GvTempsAnim_f = gpGlobals->time + 4.0;
				break;
			case 2:
				GvAnim_i = Ga_HACHE_IDLE_02;
				GvTempsAnim_f = gpGlobals->time + 4.0;
				break;
			case 3:
				GvAnim_i = Ga_HACHE_IDLE_03;
				GvTempsAnim_f = gpGlobals->time + 4.0;
				break;
			}
			SendWeaponAnim( GvAnim_i );
		}
		else
		{
			m_pPlayer->pev->viewmodel = 0; 
			m_pPlayer->pev->weaponmodel = 0;
		}
	}
}

#endif


