/*******************************\
*	WormsHL						*
*		www.wormshl.fr.st		*
*								*
*	Groove						*
*		groove@wormshl.net		*
*								*
*	arme_laser.cpp				*
*		27022002				*
\*******************************/

#include "extdll.h"
#include "util.h"
#include "cbase.h"
#include "monsters.h"
#include "player.h"
#include "generic_arme.h"
#include "generic_balle.h"
#include "arme_laser.h"
#include "gamerules.h"
#include "shake.h"

#ifdef Gd_ARME_LASER

LINK_ENTITY_TO_CLASS( weapon_laser, GcLaser );

int GcLaser::GetItemInfo( ItemInfo *p )
{
	p->pszName = STRING( pev->classname );
	p->pszAmmo1 = "munition_minitraillette";
	p->iMaxAmmo1 = Gm_MUNI_MAX;
	p->iSlot = Gm_LASER_SLOT;
	p->iPosition = Gm_LASER_POSITION;
	p->iId = m_iId = Gm_LASER_ID;
	p->iWeight = Gm_LASER_POID;
	p->iFlags = ITEM_FLAG_LIMITINWORLD | ITEM_FLAG_EXHAUSTIBLE;
	return 1;
}

void GcLaser::Spawn( void )
{
	Precache( );

	m_iId = Gm_LASER_ID;
	iGvAmmoDefault = Gm_LASER_MUNITION;

	GvTempsFDT_f = 0;
	GvNbTire_i = 0;
	GvPreci_i = 1;
	GvTempsPreci_f = 0;

	FallInit( );
}

void GcLaser::Precache( void )
{
	PRECACHE_MODEL( Gm_LASER_FICHIER_MODEL_P );
	PRECACHE_MODEL( Gm_LASER_FICHIER_MODEL_V );
	PRECACHE_MODEL( Gm_LASER_FICHIER_MODEL_R );
	GvShell_i = PRECACHE_MODEL( Gm_LASER_FICHIER_MODEL_D );
	PRECACHE_SOUND( Gm_LASER_FICHIER_SON_TIRE );
}

BOOL GcLaser::Deploy( void )
{
	GvPreci_i = 1;
	GvTempsPreci_f = 0;

	return DefaultDeploy( Gm_LASER_FICHIER_MODEL_V, Gm_LASER_FICHIER_MODEL_P, Ga_LASER_DRAW, "mitraillette" );
}

void GcLaser::Holster( void )
{
	if( m_pSpot )
	{
		m_pSpot->Killed( NULL, GIB_NEVER );
		m_pSpot = NULL;
	}

	if( !m_pPlayer->m_rgAmmo[iGvAmmoID] )
	{
		m_pPlayer->iGvArmes[Gm_LASER_SLOT] &= ~( 1<<Gm_LASER_POSITION );
		SetThink( DestroyItem );
		pev->nextthink = gpGlobals->time + 0.1;
	}

	GfFinDeTour( &GvTempsFDT_f, 0 );
}

void GcLaser::GfAttack1( void )
{
	if( !m_pPlayer->GvPlayerTPT.bTire || !m_pPlayer->m_rgAmmo[iGvAmmoID] )
	{
		GfAttack0( );
		return;
	}

	GfTire( );

	if( g_pGameRules->GvMortSubite )
		UTIL_ScreenFade( m_pPlayer, Vector( 255, 0, 0 ), 1, 0.5, 64, FFADE_OUT | FFADE_STAYOUT );
	else
		UTIL_ScreenFade( m_pPlayer, Vector( 1,1,1 ), 0.25, 0.25, 64, FFADE_IN );

	fGvAttack1 = gpGlobals->time + Gm_LASER_CADANCE_TIRE;
	fGvAttack0 = gpGlobals->time + 0.14;
}

void GcLaser::GfAttack2( void )
{
	GfAttack0( );
	fGvAttack2 = gpGlobals->time + 0.5;
}

void GcLaser::GfTire( void )
{
	if( GvPreci_i < Gm_LASER_NB_TIRE )
		GvPreci_i++;

	SendWeaponAnim( Ga_LASER_TIRE );
	m_pPlayer->SetAnimation( PLAYER_ATTACK1 );

	Vector vecSrc = m_pPlayer->GetGunPosition( ) + gpGlobals->v_forward * 16;// + gpGlobals->v_up * 8;

	// Groove - 13022002 - Cration d'une balle.
	short GvTraineCouleur[3][6] = 
	{
		{ 15,  1, 255, 128,   0, 255 },// Life, Width, r, v, b, brightness
		{ 20, 10, 255, 128,   0,  32 },// Life, Width, r, v, b, brightness
		{  0,  0,   0,   0,   0,   0 } // Life, Width, r, v, b, brightness
	};

	int GvBalleOption = Gm_BALLE_FUMEE | Gm_BALLE_BRUIT_RICOCHE | Gm_BALLE_MAKE_DOMMAGE | Gm_BALLE_ACCELERATION;

	GcGenericBalle::GfCreateBalle( vecSrc,
								   m_pPlayer->pev->angles, 
								   Gm_LASER_PRECISION * GvPreci_i / Gm_LASER_NB_TIRE, 
								   Gm_LASER_BALLE_VITESSE, 
								   Gm_LASER_DOMMAGE,
								   0.5,// Duree de vie de la balle si option active.
								   GvTraineCouleur,
								   1,// Nombre de trainees
								   GvBalleOption,//short GvOptionCreate, 
								   m_pPlayer, 
								   this, 
								   Gm_LASER_ID );

	EMIT_SOUND( ENT( m_pPlayer->pev ), CHAN_ITEM, Gm_LASER_FICHIER_SON_TIRE, 0.7, ATTN_NORM );

	// Groove - 13022002 - flash en bout de canon du minicanon.
	m_pPlayer->m_iWeaponFlash = BRIGHT_GUN_FLASH;
	m_pPlayer->pev->effects = (int)(m_pPlayer->pev->effects) | EF_MUZZLEFLASH;
	
	// Groove - 13022002 - cration d'une douille
	Vector	vecShellVelocity = m_pPlayer->pev->velocity 
							 + gpGlobals->v_right * RANDOM_FLOAT(50,70) 
							 + gpGlobals->v_up * RANDOM_FLOAT(100,150) 
							 + gpGlobals->v_forward * 25;

	EjectBrass( pev->origin + m_pPlayer->pev->view_ofs
					+ gpGlobals->v_up * -12 
					+ gpGlobals->v_forward * 20 
					+ gpGlobals->v_right * 4, vecShellVelocity, pev->angles.y, GvShell_i, TE_BOUNCE_SHELL ); 

	m_pPlayer->GfSentirDommange( Gm_LASER_SENTIR_DOMMAGE, 0 );
//	m_pPlayer->pev->velocity = m_pPlayer->pev->velocity - gpGlobals->v_forward * 32;
//	m_pPlayer->pev->velocity = m_pPlayer->pev->velocity + gpGlobals->v_right * 64;

	GvTempsPreci_f = gpGlobals->time + 0.2;
	GvNbTire_i++;
	if( GvNbTire_i >= Gm_LASER_NB_TIRE )
	{
		GvNbTire_i = 0;
		m_pPlayer->m_rgAmmo[iGvAmmoID]--;

		GvTempsAnim_f = gpGlobals->time + 1.0;
		
		m_pPlayer->GvPlayerTPT.bAmmo = true;
		m_pPlayer->GvPlayerTPT.bTire = false;
		GvTempsFDT_f = gpGlobals->time;
	}
}

void GcLaser::GfAttack0( void )
{
	fGvAttack0 = gpGlobals->time + 1.0;
	GfFinDeTour( &GvTempsFDT_f, Gm_LASER_TEMPS_DEPLACEMENT );

	if( m_pPlayer->iGvPlayerOutil & Gm_PLAYER_OUTIL_VISEE )
		UpdateSpot( );
	else if( m_pSpot &&!( m_pPlayer->iGvPlayerOutil & Gm_PLAYER_OUTIL_VISEE ))
	{
		m_pSpot->Killed( NULL, GIB_NEVER );
		m_pSpot = NULL;
	}

	if( GvNbTire_i && !m_pPlayer->GvPlayerTPT.bTire )
	{
		GvNbTire_i = 0;
		m_pPlayer->m_rgAmmo[iGvAmmoID]--;
	}

	if( GvPreci_i && GvTempsPreci_f &&( GvTempsPreci_f < gpGlobals->time ))
	{
		GvPreci_i--;
		GvTempsPreci_f = gpGlobals->time + 0.2;
	}

	if( GvTempsAnim_f &&( GvTempsAnim_f < gpGlobals->time ))
	{
		if( m_pPlayer->m_rgAmmo[iGvAmmoID] )
		{
			GvTempsAnim_f = gpGlobals->time + 2.0;
			SendWeaponAnim( Ga_LASER_IDLE );
		}
		else
		{
			m_pPlayer->pev->viewmodel = 0; 
			m_pPlayer->pev->weaponmodel = 0;
		}
	}
}

#endif