/*******************************\
*	WormsHL						*
*		www.wormshl.fr.st		*
*								*
*	Groove						*
*		groove@wormshl.net		*
*								*
*	arme_mortier.cpp			*
*		18032002				*
\*******************************/

#include "extdll.h"
#include "util.h"
#include "cbase.h"
#include "player.h"
#include "generic_arme.h"
#include "generic_balle.h"
#include "arme_mortier.h"
#include "arme_cluster.h"
#include "gamerules.h"
#include "shake.h"

#ifdef Gd_ARME_MORTIER

LINK_ENTITY_TO_CLASS( weapon_mortier, GcMortier );

int GcMortier::GetItemInfo( ItemInfo *p )
{
	p->pszName = STRING( pev->classname );
	p->pszAmmo1 = "munition_mortier";
	p->iMaxAmmo1 = Gm_MUNI_MAX;
	p->iSlot = Gm_MORTIER_SLOT;
	p->iPosition = Gm_MORTIER_POSITION;
	p->iId = m_iId = Gm_MORTIER_ID;
	p->iWeight = Gm_MORTIER_POID;
	p->iFlags = ITEM_FLAG_LIMITINWORLD | ITEM_FLAG_EXHAUSTIBLE;
	return 1;
}

void GcMortier::Spawn( void )
{
	Precache( );

	m_iId = Gm_MORTIER_ID;
	iGvAmmoDefault = Gm_MORTIER_MUNITION;

	GvTempsPuis_f = 0;
	GvTempsFDT_f = 0;

	FallInit( );
}

void GcMortier::Precache( void )
{
	PRECACHE_MODEL( Gm_MORTIER_FICHIER_MODEL_V );
	PRECACHE_MODEL( Gm_MORTIER_FICHIER_MODEL_R );
	PRECACHE_MODEL( Gm_MORTIER_FICHIER_MODEL_P );
	PRECACHE_SOUND( Gm_MORTIER_FICHIER_SON_TIRE );
}

BOOL GcMortier::Deploy( void )
{
	GvTempsAnim_f = gpGlobals->time + 2.0;
	return DefaultDeploy( Gm_MORTIER_FICHIER_MODEL_V, Gm_MORTIER_FICHIER_MODEL_P, Ga_MORTIER_DRAW, "mortier" );
}

void GcMortier::Holster( void )
{
	if( m_pSpot )
	{
		m_pSpot->Killed( NULL, GIB_NEVER );
		m_pSpot = NULL;
	}

	if( !m_pPlayer->m_rgAmmo[iGvAmmoID] )
	{
		//m_pPlayer->iGvArmes &= ~( 1<<Gm_MORTIER_ID );
		m_pPlayer->iGvArmes[Gm_MORTIER_SLOT] &= ~( 1<<Gm_MORTIER_POSITION );
		SetThink( DestroyItem );
		pev->nextthink = gpGlobals->time + 0.1;
	}

	GfFinDeTour( &GvTempsFDT_f, 0 );
}

void GcMortier::GfAttack1( void )
{
	if( !m_pPlayer->GvPlayerTPT.bTire || !m_pPlayer->m_rgAmmo[iGvAmmoID] )
	{
		GvTempsPuis_f = 0;
		GfAttack0( );
		return;
	}

	if( !GvTempsPuis_f )
	{
		GvTempsPuis_f = gpGlobals->time;
		SendWeaponAnim( Ga_MORTIER_PREPARE_TIRE );
		GfSonPuissanceOn( );
	}

	if( GfPuissanceProgressive( GvTempsPuis_f ))
		GfTire( Gm_ARME_TEMPS_TIRE_MAX );

	fGvAttack1 = fGvAttack0 = gpGlobals->time + 0.02;
}

void GcMortier::GfTire( float GvTire_f )
{
	GfPuissanceAffichage( WEAPON_PUISS_PROG_OFF, Gm_WORM_DEFAUT );
	GfSonAttack( );
	
	EMIT_SOUND( ENT( m_pPlayer->pev ), CHAN_WEAPON, Gm_MORTIER_FICHIER_SON_TIRE, 0.7, ATTN_NORM );

	// Groove - 15032002 - Animations
	SendWeaponAnim( Ga_MORTIER_TIRE );
	m_pPlayer->SetAnimation( PLAYER_ATTACK1 );

	// Groove - 15032002 - Muzzle flash
	m_pPlayer->m_iWeaponFlash = BRIGHT_GUN_FLASH;
	m_pPlayer->pev->effects = static_cast<int>(m_pPlayer->pev->effects) | EF_MUZZLEFLASH;

	Vector GvOrigin_c = m_pPlayer->GetGunPosition( ) + gpGlobals->v_forward * 16;// /*+ gpGlobals->v_right * 8*/ + gpGlobals->v_up * 8;

	short GvTraine_18i[3][6] = 
	{
		{  5,  1, 255, 255, 255, 192 },// Life, Width, r, v, b, brightness
		{ 10,  5, 128, 128, 255, 192 },// Life, Width, r, v, b, brightness
		{ 15, 10,   0,   0, 255, 128 } // Life, Width, r, v, b, brightness
	};

	// Groove - 15032002 - options de la balle pour le bazooka
	int GvBalleOption_i = Gm_BALLE_EXPLOSE | Gm_BALLE_BRIANTE | Gm_BALLE_CHUTE | Gm_BALLE_SON | Gm_BALLE_CLUSTER | Gm_BALLE_VENT;

	GcGenericBalle::GfCreateBalle( GvOrigin_c,
								   m_pPlayer->pev->angles, 
								   1, 
								   static_cast<int>( 750 * GvTire_f ),
								   Gm_MORTIER_DOMMAGE, 
								   0,// Duree de vie de la balle si option active.
								   GvTraine_18i,
								   3,// Nombre de trainees
								   GvBalleOption_i,//short GvOptionCreate, 
								   m_pPlayer, 
								   this,
								   Gm_MORTIER_ID );
	
	// Groove - 15032002 - Sentir tire
	m_pPlayer->pev->punchangle.x -= Gm_MORTIER_RECUL + static_cast<int>( Gm_MORTIER_RECUL * GvTire_f );

	if( g_pGameRules->GvMortSubite )
		UTIL_ScreenFade( m_pPlayer, Vector( 255, 0, 0 ), 1, 0.5, 64, FFADE_OUT | FFADE_STAYOUT );
	else
		UTIL_ScreenFade( m_pPlayer, Vector( 1,1,1 ), 0.25, 0.25, 64, FFADE_IN );

	m_pPlayer->m_rgAmmo[iGvAmmoID]--;

	GvTempsAnim_f = gpGlobals->time + 2.0;
	GvTempsPuis_f = 0;

	m_pPlayer->GvPlayerTPT.bAmmo = true;
	m_pPlayer->GvPlayerTPT.bTire = false;
	GvTempsFDT_f = gpGlobals->time;
}

void GcMortier::GfAttack2( void )
{
	GfAttack0( );
	fGvAttack2 = gpGlobals->time + 0.5;
}

void GcMortier::GfAttack0( void )
{
	fGvAttack0 = gpGlobals->time + 1.0;
	GfFinDeTour( &GvTempsFDT_f, Gm_MORTIER_TEMPS_DEPLACEMENT );

	if( m_pPlayer->iGvPlayerOutil & Gm_PLAYER_OUTIL_VISEE )
		UpdateSpot( );
	else if(( m_pSpot )&&!( m_pPlayer->iGvPlayerOutil & Gm_PLAYER_OUTIL_VISEE ))
	{
		m_pSpot->Killed( NULL, GIB_NEVER );
		m_pSpot = NULL;
	}

	if(GvTempsPuis_f )
	{
		// Groove - 12122001 - Un msg inutile
		if(( gpGlobals->time - GvTempsPuis_f ) < 0.2 )
			GfMsgStupide( m_pPlayer );

		if( !m_pPlayer->IsAlive( ))
			GvTempsPuis_f = 0.0;
		else 
			GfTire( gpGlobals->time - GvTempsPuis_f );
	}

	if( GvTempsAnim_f &&( GvTempsAnim_f < gpGlobals->time ))
	{
		if( m_pPlayer->m_rgAmmo[iGvAmmoID] )
		{
			SendWeaponAnim( Ga_MORTIER_IDLE );
			GvTempsAnim_f = gpGlobals->time + 2.0;
		}
		else
		{
			m_pPlayer->pev->viewmodel = 0; 
			m_pPlayer->pev->weaponmodel = 0;
		}
	}
}

#endif