/*******************************\
*	WormsHL						*
*		www.wormshl.fr.st		*
*								*
*	Groove						*
*		groove@wormshl.net		*
*								*
*	arme_teleporteur.cpp		*
*		19022002				*
\*******************************/

#include "extdll.h"
#include "util.h"
#include "cbase.h"
#include "customentity.h"
#include "shake.h"
#include "fx_beam.h"
#include "player.h"
#include "generic_arme.h"
#include "arme_teleporteur.h"
#include "gamerules.h"

#ifdef Gd_ARME_TELEPORTEUR

LINK_ENTITY_TO_CLASS( weapon_teleporteur, GcTeleporteur );

int GcTeleporteur::GetItemInfo( ItemInfo *p )
{
	p->pszName = STRING( pev->classname );
	p->pszAmmo1 = "munition_teleporteur";
	p->iMaxAmmo1 = Gm_MUNI_MAX;
	p->iSlot = Gm_TELEPORTEUR_SLOT;
	p->iPosition = Gm_TELEPORTEUR_POSITION;
	p->iId = Gm_TELEPORTEUR_ID;
	p->iWeight = Gm_TELEPORTEUR_POID;
	p->iFlags = ITEM_FLAG_LIMITINWORLD | ITEM_FLAG_EXHAUSTIBLE;
	return 1;
}

void GcTeleporteur::Spawn( void )
{
	Precache( );

	m_iId = Gm_TELEPORTEUR_ID;
	iGvAmmoDefault = Gm_TELEPORTEUR_MUNITION;
	
	GvTempsFDT_f = 0;

	FallInit( );
}

void GcTeleporteur::Precache( void )
{
	PRECACHE_MODEL( Gm_TELEPORTEUR_FICHIER_MODEL_P );
	PRECACHE_MODEL( Gm_TELEPORTEUR_FICHIER_MODEL_V );
    PRECACHE_MODEL( Gm_TELEPORTEUR_FICHIER_SPRITE_TELEPORT );
	PRECACHE_MODEL( Gm_TELEPORTEUR_FICHIER_SPRITE_ZAP );
	PRECACHE_SOUND( Gm_TELEPORTEUR_FICHIER_SON_TELEPORT );	
	PRECACHE_SOUND( Gm_TELEPORTEUR_FICHIER_SON_ZAP );
}

BOOL GcTeleporteur::Deploy( void )
{
	GvTempsVieEffect_f = 0;
	return DefaultDeploy( Gm_TELEPORTEUR_FICHIER_MODEL_V, Gm_TELEPORTEUR_FICHIER_MODEL_P, Ga_TELEPORTEUR_DRAW, "teleporteur" );
}

void GcTeleporteur::Holster( void )
{
	if( m_pSpot )
	{
		m_pSpot->Killed( NULL, GIB_NEVER );
		m_pSpot = NULL;
	}

	DestroyEffect( );

	if( !m_pPlayer->m_rgAmmo[iGvAmmoID] )
	{
		m_pPlayer->iGvArmes[Gm_TELEPORTEUR_SLOT] &= ~( 1<<Gm_TELEPORTEUR_POSITION );
		SetThink( DestroyItem );
		pev->nextthink = gpGlobals->time + 0.1;
	}
	
	GfFinDeTour( &GvTempsFDT_f, 0 );
}


void GcTeleporteur::GfAttack1( void ) 
{
	if( !m_pPlayer->GvPlayerTPT.bTire || !m_pPlayer->m_rgAmmo[iGvAmmoID] )
	{
		GfAttack0( );
		return;
	}

//	SendWeaponAnim( TELEPORTEUR_TIRE ); // Groove - 08032002 - au vue de la qualit de l'anim de tire ...

	GfTire( m_pPlayer->GetGunPosition( ), gpGlobals->v_forward );
} 

void GcTeleporteur::GfTire( const Vector &GvOrigin_c, const Vector &GvDirection_c )
{
	EMIT_SOUND( ENT( m_pPlayer->pev ), CHAN_WEAPON, Gm_TELEPORTEUR_FICHIER_SON_ZAP, 1, ATTN_NORM );

	m_pPlayer->m_iWeaponFlash = BRIGHT_GUN_FLASH;
	m_pPlayer->pev->effects = (int)(m_pPlayer->pev->effects) | EF_MUZZLEFLASH;
	
	Vector GvDestination_c = GvOrigin_c + GvDirection_c * 2048;
	edict_t		*pentIgnore;
	TraceResult tr;

	pentIgnore = m_pPlayer->edict( );

	UTIL_TraceLine( GvOrigin_c, GvDestination_c, dont_ignore_monsters, pentIgnore, &tr );

	UpdateEffect( GvOrigin_c, tr.vecEndPos, 3.0 );
	
	UTIL_MakeVectors( m_pPlayer->pev->v_angle ); 
	Vector vecEnd = GvOrigin_c + gpGlobals->v_forward * 8192; 
	UTIL_TraceLine( GvOrigin_c, vecEnd, dont_ignore_monsters, ENT( m_pPlayer->pev ), & tr ); 
   
	CBaseEntity *pHit = CBaseEntity::Instance(tr.pHit);  
    if( pHit && FClassnameIs( pHit->pev,"player" )) 
	{      
		vec3_t hold;
		hold =   m_pPlayer->pev->origin ;          
		m_pPlayer->pev->origin = pHit->pev->origin; 
		pHit->pev->origin =hold; 


		EMIT_SOUND( ENT( m_pPlayer->pev ), CHAN_WEAPON, Gm_TELEPORTEUR_FICHIER_SON_TELEPORT, 1, ATTN_NORM);
		EMIT_SOUND( ENT( pHit->pev ), CHAN_WEAPON, Gm_TELEPORTEUR_FICHIER_SON_TELEPORT, 1, ATTN_NORM);

		if( g_pGameRules->GvMortSubite )
			UTIL_ScreenFade( m_pPlayer, Vector( 255, 0, 0 ), 1, 0.5, 64, FFADE_OUT | FFADE_STAYOUT );
		else
			UTIL_ScreenFade( m_pPlayer, Vector( 0,64,192), 0.5, 0.5, 128, FFADE_IN );

		m_pPlayer->m_rgAmmo[iGvAmmoID]--;

		m_pPlayer->GvPlayerTPT.bTire = false;
		GvTempsFDT_f = gpGlobals->time;
	} 
	else
	{
		GfMsgStupide( m_pPlayer );
	}
	fGvAttack0 = fGvAttack1 = fGvAttack2 = gpGlobals->time + 1.0;
}

void GcTeleporteur::GfAttack2( void )
{
	if( !m_pPlayer->GvPlayerTPT.bTire || !m_pPlayer->m_rgAmmo[iGvAmmoID] )
	{
		GfAttack0( );
		return;
	}
	
	m_pPlayer->m_iWeaponFlash = BRIGHT_GUN_FLASH;
	m_pPlayer->pev->effects = (int)(m_pPlayer->pev->effects) | EF_MUZZLEFLASH;
	
	EMIT_SOUND( ENT( m_pPlayer->pev ), CHAN_WEAPON, Gm_TELEPORTEUR_FICHIER_SON_TELEPORT, 1, ATTN_NORM);

	if( g_pGameRules->GvMortSubite )
		UTIL_ScreenFade( m_pPlayer, Vector( 255, 0, 0 ), 1, 0.5, 64, FFADE_OUT | FFADE_STAYOUT );
	else
		UTIL_ScreenFade( m_pPlayer, Vector( 0,64,192), 0.5, 0.5, 128, FFADE_IN );
	
	g_pGameRules->GetPlayerSpawnSpot( m_pPlayer );

	m_pPlayer->m_rgAmmo[iGvAmmoID]--;
	
	m_pPlayer->GvPlayerTPT.bTire = false;
	GvTempsFDT_f = gpGlobals->time;

	fGvAttack2 = gpGlobals->time + 0.5;
}

void GcTeleporteur::UpdateEffect( const Vector &startPoint, const Vector &endPoint, float timeBlend )
{
	if( !pEffect )
		CreateEffect();

	pEffect->SetStartPos( endPoint );
}

void GcTeleporteur::CreateEffect( void )
{
	DestroyEffect( );
	GvTempsVieEffect_f = gpGlobals->time + 1.0;
	pEffect = CBeam::BeamCreate( Gm_TELEPORTEUR_FICHIER_SPRITE_ZAP, 55 );
	pEffect->PointEntInit( m_pPlayer->GetGunPosition( ) + gpGlobals->v_up * 32 + gpGlobals->v_right * 32, m_pPlayer->entindex() );
	pEffect->SetScrollRate( 25 );
	pEffect->SetBrightness( 100 );
	pEffect->SetEndAttachment( 1 );
	pEffect->pev->spawnflags |= SF_BEAM_TEMPORARY;

	pEffect->SetColor( 50, 255, 50 );
	pEffect->SetNoise( 8 );
}


void GcTeleporteur::DestroyEffect( void )
{
	if( pEffect )
	{
		UTIL_Remove( pEffect );
		pEffect = NULL;
	}
}

void GcTeleporteur::GfAttack0( void )
{
	fGvAttack0 = gpGlobals->time + 1.0;
	GfFinDeTour( &GvTempsFDT_f, Gm_TELEPORTEUR_TEMPS_DEPLACEMENT );

	if( m_pPlayer->iGvPlayerOutil & Gm_PLAYER_OUTIL_VISEE )
		UpdateSpot( );
	else if(( m_pSpot )&&!( m_pPlayer->iGvPlayerOutil & Gm_PLAYER_OUTIL_VISEE ))
	{
		m_pSpot->Killed( NULL, GIB_NEVER );
		m_pSpot = NULL;
	}

	if(( GvTempsVieEffect_f < gpGlobals->time )&&( GvTempsVieEffect_f != 0 ))
	{
		GvTempsVieEffect_f = 0;
		DestroyEffect( );
	}

	if( fGvAttack0 < gpGlobals->time )
	{
		SendWeaponAnim( Ga_TELEPORTEUR_IDLE1 );
		fGvAttack0 = gpGlobals->time + 16.0;
	}

	if( !m_pPlayer->m_rgAmmo[iGvAmmoID] )
	{
		m_pPlayer->pev->viewmodel = 0; 
		m_pPlayer->pev->weaponmodel = 0;
	}
}

#endif