/*******************************\
*	WormsHL						*
*		www.wormshl.fr.st		*
*								*
*	Groove						*
*		groove@wormshl.net		*
*								*
*	fx_explode.cpp				*
*		08052002				*
\*******************************/

#include "extdll.h"
#include "util.h"
#include "cbase.h"
#include "fx_decals.h"
#include "fx_explode.h"

LINK_ENTITY_TO_CLASS( env_explosion, CEnvExplosion );

void CEnvExplosion::KeyValue( KeyValueData *pkvd )
{
	if( FStrEq(pkvd->szKeyName, "iMagnitude" ))
	{
		m_iMagnitude = atoi( pkvd->szValue );
		pkvd->fHandled = TRUE;
	}
	else
		CBaseEntity::KeyValue( pkvd );
}

void CEnvExplosion::Spawn( void )
{ 
	pev->solid = SOLID_NOT;
	pev->effects = EF_NODRAW;
	pev->movetype = MOVETYPE_NONE;
}

void CEnvExplosion::Use( CBaseEntity *pActivator, CBaseEntity *pCaller, USE_TYPE useType, float value )
{ 
	TraceResult tr;

	pev->model = iStringNull;//invisible
	pev->solid = SOLID_NOT;// intangible

	Vector		vecSpot;// trace starts here!

	vecSpot = pev->origin + Vector ( 0 , 0 , 8 );
	
	UTIL_TraceLine ( vecSpot, vecSpot + Vector ( 0, 0, -40 ),  ignore_monsters, ENT(pev), & tr);
	
	// Pull out of the wall a bit
	if( tr.flFraction != 1.0 )
		pev->origin = tr.vecEndPos + (tr.vecPlaneNormal * (m_iMagnitude - 24) * 0.6);
	else
		pev->origin = pev->origin;

	int iGvDecalChoix = 0;

	if(!( pev->spawnflags & SF_ENVEXPLOSION_NODECAL ))
	{
		if( m_iMagnitude <= 5 )
			iGvDecalChoix = DECAL_EXPLOSE_005;
		else if( m_iMagnitude <= 10 )
			iGvDecalChoix = DECAL_EXPLOSE_010;
		else if( m_iMagnitude <= 25 )
			iGvDecalChoix = DECAL_EXPLOSE_025;
		else if( m_iMagnitude <= 50 )
			iGvDecalChoix = DECAL_EXPLOSE_050;
		else if( m_iMagnitude <= 75 )
			iGvDecalChoix = DECAL_EXPLOSE_075;
		else if( m_iMagnitude <= 100 )
			iGvDecalChoix = DECAL_EXPLOSE_100;
		else if( m_iMagnitude <= 150 )
			iGvDecalChoix = DECAL_EXPLOSE_150;
		else if( m_iMagnitude <= 200 )
			iGvDecalChoix = DECAL_EXPLOSE_200;
		else if( m_iMagnitude <= 300 )
			iGvDecalChoix = DECAL_EXPLOSE_300;

		UTIL_DecalTrace( &tr, iGvDecalChoix );
	}

	MESSAGE_BEGIN( MSG_PAS, SVC_TEMPENTITY, pev->origin );
		WRITE_BYTE( TE_EXPLOSION);
		WRITE_COORD( pev->origin.x );
		WRITE_COORD( pev->origin.y );
		WRITE_COORD( pev->origin.z );
		WRITE_SHORT( g_sModelIndexFireball );
		WRITE_BYTE( 0 ); // no sprite
		WRITE_BYTE( 15  ); // framerate
		WRITE_BYTE( TE_EXPLFLAG_NONE );
	MESSAGE_END( );
	/*
	// draw fireball
	if ( !( pev->spawnflags & SF_ENVEXPLOSION_NOFIREBALL ))
	{
		MESSAGE_BEGIN( MSG_PAS, SVC_TEMPENTITY, pev->origin );
			WRITE_BYTE( TE_EXPLOSION);
			WRITE_COORD( pev->origin.x );
			WRITE_COORD( pev->origin.y );
			WRITE_COORD( pev->origin.z );
			WRITE_SHORT( g_sModelIndexFireball );
			WRITE_BYTE( m_iMagnitude / Gm_SPRITE_EXPLOSE_DIV + Gm_SPRITE_EXPLOSE_PLUS );
//			WRITE_BYTE( (BYTE)m_spriteScale ); // scale * 10
			WRITE_BYTE( 15  ); // framerate
			WRITE_BYTE( TE_EXPLFLAG_NONE );
		MESSAGE_END();
	}
	else
	{
		MESSAGE_BEGIN( MSG_PAS, SVC_TEMPENTITY, pev->origin );
			WRITE_BYTE( TE_EXPLOSION);
			WRITE_COORD( pev->origin.x );
			WRITE_COORD( pev->origin.y );
			WRITE_COORD( pev->origin.z );
			WRITE_SHORT( g_sModelIndexFireball );
			WRITE_BYTE( 0 ); // no sprite
			WRITE_BYTE( 15  ); // framerate
			WRITE_BYTE( TE_EXPLFLAG_NONE );
		MESSAGE_END();
	}*/

	if( !( pev->spawnflags & SF_ENVEXPLOSION_NODAMAGE ))
		RadiusDamage ( pev, pev, m_iMagnitude, CLASS_NONE, Gm_DMG_EXPLOSION );

	SetThink( Smoke );
	pev->nextthink = gpGlobals->time + 0.3;
}

void CEnvExplosion::Smoke( void )
{
#ifdef Gd_EXPLOSION_FUMEE
	if ( !( pev->spawnflags & SF_ENVEXPLOSION_NOSMOKE ) )
	{
		MESSAGE_BEGIN( MSG_PAS, SVC_TEMPENTITY, pev->origin );
			WRITE_BYTE( TE_SMOKE );
			WRITE_COORD( pev->origin.x );
			WRITE_COORD( pev->origin.y );
			WRITE_COORD( pev->origin.z );
			WRITE_SHORT( g_sModelIndexSmoke );
			WRITE_BYTE( m_iMagnitude + / Gm_SPRITE_FUMEE_DIV + Gm_SPRITE_FUMEE_PLUS );
//			WRITE_BYTE( (BYTE)m_spriteScale ); // scale * 10
			WRITE_BYTE( 12  ); // framerate
		MESSAGE_END();
	}
#endif
	
	if(!( pev->spawnflags & SF_ENVEXPLOSION_REPEATABLE ))
		UTIL_Remove( this );
}

// Groove - 09112001 - explosion, prise en compte de 'magnitude' pour explosion proposionnel aux degats
void ExplosionCreate( const Vector &center, const Vector &angles, edict_t *pOwner, int magnitude, BOOL doDamage )
{
	KeyValueData	kvd;
	char			buf[128];

	CBaseEntity *pExplosion = CBaseEntity::Create( "env_explosion", center, angles, pOwner );
	sprintf( buf, "%3d", magnitude );
	kvd.szKeyName = "iMagnitude";
	kvd.szValue = buf;
	pExplosion->KeyValue( &kvd );
	if( !doDamage )
		pExplosion->pev->spawnflags |= SF_ENVEXPLOSION_NODAMAGE;

//	pExplosion->GvDegat = magnitude;
	pExplosion->Spawn();
	pExplosion->Use( NULL, NULL, USE_TOGGLE, 0 );
}
