/*******************************\
*	WormsHL						*
*		www.wormshl.fr.st		*
*								*
*	Groove						*
*		groove@wormshl.net		*
*								*
*	gamerules.h					*
*		25042002				*
\*******************************/

#ifndef GAMERULES_H
#define GAMERULES_H

class CBasePlayerItem;
class CBasePlayer;
class CItem;

// Groove - 03112001 - Identification des diffrents types de mode de jeux
#define Gm_GAMERULES_NULL		0 // Non fonctionnel
#define Gm_GAMERULES_NORMAL		1
#define Gm_GAMERULES_BATTE		2
#define Gm_GAMERULES_GROUP		3
#define Gm_GAMERULES_ZONE		4
#define Gm_GAMERULES_CONTROLE	5
#define Gm_GAMERULES_BARIL		6
#define Gm_GAMERULES_ACTION		7

// Groove - 28022002 - Pour le spawn d'entit
#define Gm_ENTITY_NULL			0
#define Gm_ENTITY_BARIL			1
#define Gm_ENTITY_MINE			2
#define Gm_ENTITY_BARIL_BLEU	3
#define Gm_ENTITY_BARIL_ROUGE	4
#define Gm_ENTITY_COMETE		5

/*
// weapon respawning return codes
enum
{	
	GR_NONE = 0,
	
	GR_WEAPON_RESPAWN_YES,
	GR_WEAPON_RESPAWN_NO,
	
	GR_AMMO_RESPAWN_YES,
	GR_AMMO_RESPAWN_NO,
	
	GR_ITEM_RESPAWN_YES,
	GR_ITEM_RESPAWN_NO,

	GR_PLR_DROP_GUN_ALL,
	GR_PLR_DROP_GUN_ACTIVE,
	GR_PLR_DROP_GUN_NO,

	GR_PLR_DROP_AMMO_ALL,
	GR_PLR_DROP_AMMO_ACTIVE,
	GR_PLR_DROP_AMMO_NO,
};*/

// Player relationship return codes
enum
{
	GR_NOTTEAMMATE = 0,
	GR_TEAMMATE,
	GR_ENEMY,
	GR_ALLY,
	GR_NEUTRAL,
};

enum GeScore
{
	Gm_SCORE_BLEU_1 = 0,
	Gm_SCORE_ROUGE_1,
	Gm_SCORE_BLEU_2,
	Gm_SCORE_ROUGE_2,
	Gm_SCORE_BLEU_3,
	Gm_SCORE_ROUGE_3,
	Gm_SCORE_INI,
	Gm_SCORE_INI_FULL
};

class CGameRules
{
public:
	virtual void Think( void ) = 0;// GR_Think - runs every server frame, should handle any timer tasks, periodic events, etc.

	virtual BOOL FShouldSwitchWeapon( CBasePlayer *pPlayer, CBasePlayerItem *pWeapon ) = 0;// should the player switch to this weapon?

// Functions to verify the single/multiplayer status of a game
	virtual BOOL IsMultiplayer( void ) = 0;// is this a multiplayer game? (either coop or deathmatch)
	virtual BOOL IsDeathmatch( void ) = 0;//is this a deathmatch game?
	virtual BOOL IsTeamplay( void ) { return FALSE; };// is this deathmatch game being played with team rules?
	
// Client connection/disconnection
	virtual BOOL ClientConnected( edict_t *pEntity, const char *pszName, const char *pszAddress, char szRejectReason[ 128 ] ) = 0;// a client just connected to the server (player hasn't spawned yet)
	virtual void InitHUD( CBasePlayer *pl ) = 0;		// the client dll is ready for updating
	virtual void ClientDisconnected( edict_t *pClient ) = 0;// a client just disconnected from the server
	virtual void UpdateGameMode( CBasePlayer *pPlayer ) {}  // the client needs to be informed of the current game mode

// Client damage rules
	virtual float FlPlayerFallDamage( CBasePlayer *pPlayer ) = 0;// this client just hit the ground after a fall. How much damage?

// Client spawn/respawn control
	virtual void PlayerSpawn( CBasePlayer *pPlayer ) = 0;// called by CBasePlayer::Spawn just before releasing player into the game
	virtual void PlayerThink( CBasePlayer *pPlayer ) = 0; // called by CBasePlayer::PreThink every frame, before physics are run and after keys are accepted
	virtual edict_t *GetPlayerSpawnSpot( CBasePlayer *pPlayer );// Place this player on their spawnspot and face them the proper direction.

	virtual BOOL ClientCommand( CBasePlayer *pPlayer, const char *pcmd ) { return FALSE; };  // handles the user commands;  returns TRUE if command handled properly
	virtual void ClientUserInfoChanged( CBasePlayer *pPlayer, char *infobuffer ) {}		// the player has changed userinfo;  can change it now

// Client kills/scoring
	virtual int IPointsForKill( CBasePlayer *pAttacker, CBasePlayer *pKilled ) = 0;// how many points do I award whoever kills this player?
	virtual void PlayerKilled( CBasePlayer *pVictim, entvars_t *pKiller, entvars_t *pInflictor ) = 0;// Called each time a player dies
	virtual void DeathNotice( CBasePlayer *pVictim, entvars_t *pKiller, entvars_t *pInflictor )=  0;// Call this from within a GameRules class to report an obituary.

// Ammo retrieval
	virtual BOOL CanHaveAmmo( CBasePlayer *pPlayer, const char *pszAmmoName, int iMaxCarry );// can this player take more of this ammo?

// Teamplay stuff
	virtual const char *GetTeamID( CBaseEntity *pEntity ) = 0;// what team is this entity on?
	virtual int PlayerRelationship( CBaseEntity *pPlayer, CBaseEntity *pTarget ) = 0;// What is the player's relationship with this entity?
	virtual int GetTeamIndex( const char *pTeamName ) { return -1; }
	virtual const char *GetIndexedTeamName( int teamIndex ) { return ""; }
	virtual BOOL IsValidTeam( const char *pTeamName ) { return TRUE; }
	virtual void ChangePlayerTeam( CBasePlayer *pPlayer, const char *pTeamName, BOOL bKill, BOOL bGib ) {}
	virtual const char *SetDefaultPlayerTeam( CBasePlayer *pPlayer ) { return ""; }

// Sounds
	virtual BOOL PlayTextureSounds( void ) { return TRUE; }
	virtual BOOL PlayFootstepSounds( CBasePlayer *pl, float fvol ) { return TRUE; }

// Monsters

	// Immediately end a multiplayer game
	virtual void EndMultiplayerGame( void ){ }

	virtual void GfGestionScore( void ){ }
	void GfScoreAdd( GeScore GvScore );
	// Groove - 28022002 - Bonus spawn entit ( mine / baril ).
	void GfEntitySpawn_i( int iGvChoixEntity );
	CBaseEntity *pEntity;

	// Groove - 25042002 - Proprit des gamerules.
	// Groove - 22112001 - Indique le mode de jeu en court
	virtual short iGfGamerulesActif( void ){ return Gm_GAMERULES_NULL; } // Pas de mode de jeu choisi
	// Groove - 22112001 - defini si le spawn des caisses est actif ou non
	virtual bool bGfCaisseCanSpawn( void ){ return true; } // Chutes de caisses par defaut
	// Groove - 25042002 - Marque des points avec les kills
	virtual bool bGfScoreKill( void ){ return true; }
	// Groove - 25042002 - Spawn par group ou commun
	virtual bool bGfSpawnGroup( void ){ return false; }
	// Groove - 13122001 - Spawn des tombes
	virtual bool bGfTombeCanSpawn( void ){ return true; }

	// Groove - 22042002 - Tour par tour
	virtual void GfTempsTPT( CBasePlayer *pPlayer );
	virtual void GfFinDeTourAutoTPT( CBasePlayer *pPlayer );
	virtual short GfCompteWormsTPT( void );
	virtual void GfMaladieGestion( CBasePlayer *pPlayer );
	virtual void GfWormsBleuxTPT( CBasePlayer *pPlayer );
	virtual void GfWormsRougesTPT( CBasePlayer *pPlayer );
	virtual void GfCaisseGestion( void );
	void GfIniTPT( void );
	void GfWaterUpdate( void );
	void GfVentUpdate( void );
	void GfMortSubiteControle( CBasePlayer *pGvPlayer );

	bool GvChangeVue;
	bool GvMortSubite;

	bool GvWormsRougeTPT;
	bool GvWormsBleuTPT;
	
	// Groove - 21042002 - Le temps du TPT
	float fGvTempsAuto;
	float fGvTempsTPT;
	float fGvTempsChangeTourTPT;
	int iGvTempsTPT;

	int iGvWaterUp;
	float fGvTempsWaterUp;
	Vector GvVent;

	short iGvScoreBleuPoint;
	short iGvScoreRougePoint;
	short iGvScoreBleuManche;
	short iGvScoreRougeManche;

private:
	CBaseEntity *pGvCaisse;
	CBaseEntity *pGvWater;

	int GvNbJoueurTPT;
};

extern CGameRules *InstallGameRules( void );

// Groove - 22112001 - Regle essenciel pour le jeu multijoueur
class CHalfLifeMultiplay : public CGameRules
{
public:
	CHalfLifeMultiplay( );

// GR_Think
	virtual void Think( void );

	virtual BOOL FShouldSwitchWeapon( CBasePlayer *pPlayer, CBasePlayerItem *pWeapon );

// Functions to verify the single/multiplayer status of a game
	virtual BOOL IsMultiplayer( void );
	virtual BOOL IsDeathmatch( void );

// Client connection/disconnection
	// If ClientConnected returns FALSE, the connection is rejected and the user is provided the reason specified in
	//  svRejectReason
	// Only the client's name and remote address are provided to the dll for verification.
	virtual BOOL ClientConnected( edict_t *pEntity, const char *pszName, const char *pszAddress, char szRejectReason[ 128 ] );
	virtual void InitHUD( CBasePlayer *pl );		// the client dll is ready for updating
	virtual void ClientDisconnected( edict_t *pClient );
	virtual void UpdateGameMode( CBasePlayer *pPlayer );  // the client needs to be informed of the current game mode

// Client damage rules
	virtual float FlPlayerFallDamage( CBasePlayer *pPlayer );

// Client spawn/respawn control
	virtual void PlayerSpawn( CBasePlayer *pPlayer );
	virtual void PlayerThink( CBasePlayer *pPlayer );
	virtual edict_t *GetPlayerSpawnSpot( CBasePlayer *pPlayer );

// Client kills/scoring
	virtual int IPointsForKill( CBasePlayer *pAttacker, CBasePlayer *pKilled );
	virtual void PlayerKilled( CBasePlayer *pVictim, entvars_t *pKiller, entvars_t *pInflictor );
	virtual void DeathNotice( CBasePlayer *pVictim, entvars_t *pKiller, entvars_t *pInflictor );

// Teamplay stuff	
	virtual const char *GetTeamID( CBaseEntity *pEntity ) {return "";}
	virtual int PlayerRelationship( CBaseEntity *pPlayer, CBaseEntity *pTarget );

	virtual BOOL PlayTextureSounds( void ) { return FALSE; }
	virtual BOOL PlayFootstepSounds( CBasePlayer *pl, float fvol );

	// Immediately end a multiplayer game
	virtual void EndMultiplayerGame( void ) { GoToIntermission(); }
	
// Groove - 27102001 - Tout le traitement des scores !
	virtual void GfGestionScore( void );
	// Groove - 21122001 - Pour supprimer les moutons si le joueur meurt
	void GfDeactivateMouton( CBasePlayer *pOwner );

	float fGvChangementDeCarte; // Pour laisser X seconde avant le changement de cartes.
//	float GvTempsScore;
	
	bool GvActiveMine_b;
	bool GvActiveBaril_b;
	bool GvActiveZoneBleu_b;
	bool GvActiveZoneRouge_b;

protected:
	virtual void ChangeLevel( void );
	virtual void GoToIntermission( void );
//	float m_flIntermissionEndTime;
	BOOL m_iEndIntermissionButtonHit;
	void SendMOTDToClient( edict_t *client );

//private:
//	CBaseEntity *GpCaisse;
};

extern DLL_GLOBAL CGameRules*	g_pGameRules;

#endif // GAMERULES_H