/*******************************\
*	WormsHL						*
*		www.wormshl.fr.st		*
*								*
*	Groove						*
*		groove@wormshl.net		*
*								*
*	generic_arme.cpp			*
*		17032002				*
\*******************************/

#include "extdll.h"
#include "util.h"
#include "cbase.h"
#include "player.h"
#include "monsters.h"
#include "generic_arme.h"
#include "generic_grenade.h"
#include "generic_balle.h"
#include "arme_snipe.h"
#include "generic_spot.h"
#include "define_slot.h"
#include "gamerules.h"
#include "nodes.h"
#include "fx_decals.h"
#include "shake.h"

//extern CGraph	WorldGraph;

DLL_GLOBAL	short	g_sModelIndexLaser;// holds the index for the laser beam
DLL_GLOBAL	short	g_sModelIndexFireball;// holds the index for the fireball
DLL_GLOBAL	short	g_sModelIndexSmoke;// holds the index for the smoke cloud
DLL_GLOBAL	short	g_sModelIndexWExplosion;// holds the index for the underwater explosion
DLL_GLOBAL	short	g_sModelIndexBubbles;// holds the index for the bubbles model

ItemInfo CBasePlayerItem::ItemInfoArray[MAX_WEAPONS];
AmmoInfo CBasePlayerItem::AmmoInfoArray[MAX_AMMO_SLOTS];

MULTIDAMAGE gMultiDamage;

//
// ClearMultiDamage - resets the global multi damage accumulator
//
void ClearMultiDamage(void)
{
	gMultiDamage.pEntity = NULL;
	gMultiDamage.amount	= 0;
	gMultiDamage.type = 0;
}


//
// ApplyMultiDamage - inflicts contents of global multi damage register on gMultiDamage.pEntity
//
// GLOBALS USED:
//		gMultiDamage

void ApplyMultiDamage(entvars_t *pevInflictor, entvars_t *pevAttacker )
{
	Vector		vecSpot1;//where blood comes from
	Vector		vecDir;//direction blood should go
	TraceResult	tr;
	
	if ( !gMultiDamage.pEntity )
		return;

	gMultiDamage.pEntity->TakeDamage(pevInflictor, pevAttacker, gMultiDamage.amount, gMultiDamage.type );
}


// GLOBALS USED:
//		gMultiDamage

void AddMultiDamage( entvars_t *pevInflictor, CBaseEntity *pEntity, float flDamage, int bitsDamageType)
{
	if ( !pEntity )
		return;
	
	gMultiDamage.type |= bitsDamageType;

	if( pEntity != gMultiDamage.pEntity )
	{
		ApplyMultiDamage(pevInflictor,pevInflictor); // UNDONE: wrong attacker!
		gMultiDamage.pEntity	= pEntity;
		gMultiDamage.amount		= 0;
	}

	gMultiDamage.amount += flDamage;
}

//
// EjectBrass - tosses a brass shell from passed origin at passed velocity
//
void EjectBrass ( const Vector &vecOrigin, const Vector &vecVelocity, float rotation, int model, int soundtype )
{
	// FIX: when the player shoots, their gun isn't in the same position as it is on the model other players see.

	MESSAGE_BEGIN( MSG_PVS, SVC_TEMPENTITY, vecOrigin );
		WRITE_BYTE( TE_MODEL);
		WRITE_COORD( vecOrigin.x);
		WRITE_COORD( vecOrigin.y);
		WRITE_COORD( vecOrigin.z);
		WRITE_COORD( vecVelocity.x);
		WRITE_COORD( vecVelocity.y);
		WRITE_COORD( vecVelocity.z);
		WRITE_ANGLE( rotation );
		WRITE_SHORT( model );
		WRITE_BYTE ( soundtype);
		WRITE_BYTE ( 25 );// 2.5 seconds
	MESSAGE_END();
}

int giAmmoIndex = 0;

// Precaches the ammo and queues the ammo info for sending to clients
void AddAmmoNameToAmmoRegistry( const char *szAmmoname )
{
	// make sure it's not already in the registry
	for ( int i = 0; i < MAX_AMMO_SLOTS; i++ )
	{
		if ( !CBasePlayerItem::AmmoInfoArray[i].pszName)
			continue;

		if ( stricmp( CBasePlayerItem::AmmoInfoArray[i].pszName, szAmmoname ) == 0 )
			return; // ammo already in registry, just quite
	}


	giAmmoIndex++;
	ASSERT( giAmmoIndex < MAX_AMMO_SLOTS );
	if( giAmmoIndex >= MAX_AMMO_SLOTS )
		giAmmoIndex = 0;

	CBasePlayerItem::AmmoInfoArray[giAmmoIndex].pszName = szAmmoname;
	CBasePlayerItem::AmmoInfoArray[giAmmoIndex].iId = giAmmoIndex;   // yes, this info is redundant
}


// Precaches the weapon and queues the weapon info for sending to clients
void UTIL_PrecacheOtherWeapon( const char *szClassname )
{
	edict_t	*pent;

	pent = CREATE_NAMED_ENTITY( MAKE_STRING( szClassname ));
	if( FNullEnt( pent ))
	{
		ALERT( at_console, "NULL Ent in UTIL_PrecacheOtherWeapon\n" );
		return;
	}
	
	CBaseEntity *pEntity = CBaseEntity::Instance (VARS( pent ));

	if( pEntity )
	{
		ItemInfo II;
		pEntity->Precache( );
		memset( &II, 0, sizeof II );
		if(((CBasePlayerItem*)pEntity)->GetItemInfo( &II ))
		{
			CBasePlayerItem::ItemInfoArray[II.iId] = II;

			if( II.pszAmmo1 && *II.pszAmmo1 )
				AddAmmoNameToAmmoRegistry( II.pszAmmo1 );

			memset( &II, 0, sizeof II );
		}
	}

	REMOVE_ENTITY( pent );
}

// called by worldspawn
void W_Precache(void)
{
	memset( CBasePlayerItem::ItemInfoArray, 0, sizeof( CBasePlayerItem::ItemInfoArray) );
	memset( CBasePlayerItem::AmmoInfoArray, 0, sizeof( CBasePlayerItem::AmmoInfoArray) );
	giAmmoIndex = 0;

	// Groove - 20122001 - Precache des armes
	#ifdef Gd_ARME_AIR_STRIKE
		UTIL_PrecacheOtherWeapon( "weapon_air_strike" );
	#endif
	#ifdef Gd_ARME_BATTE
		UTIL_PrecacheOtherWeapon( "weapon_batte" );
	#endif
	#ifdef Gd_ARME_BANANE
		UTIL_PrecacheOtherWeapon( "weapon_banane" );
	#endif
	#ifdef Gd_ARME_BAZOOKA
		UTIL_PrecacheOtherWeapon( "weapon_bazooka" );
	#endif
	#ifdef Gd_ARME_CAMERA
		UTIL_PrecacheOtherWeapon( "weapon_camera" );
	#endif
	#ifdef Gd_ARME_CLUSTER
		UTIL_PrecacheOtherWeapon( "weapon_cluster" );
	#endif
	#ifdef Gd_ARME_CROIX
		UTIL_PrecacheOtherWeapon( "weapon_croix" );
	#endif
	#ifdef Gd_ARME_DYNAMITE
		UTIL_PrecacheOtherWeapon( "weapon_dynamite" );
	#endif
	#ifdef Gd_ARME_FUSIL
		UTIL_PrecacheOtherWeapon( "weapon_fusil" );
	#endif
	#ifdef Gd_ARME_GRAPPIN
		UTIL_PrecacheOtherWeapon( "weapon_grappin" );
	#endif
	#ifdef Gd_ARME_GRENADE
		UTIL_PrecacheOtherWeapon( "weapon_grenade" );
	#endif
	#ifdef Gd_ARME_HACHE
		UTIL_PrecacheOtherWeapon( "weapon_hache" );
	#endif
	#ifdef Gd_ARME_HELICO
		UTIL_PrecacheOtherWeapon( "weapon_helico" );
	#endif
	#ifdef Gd_ARME_JETPACK
		UTIL_PrecacheOtherWeapon( "weapon_jetpack" );
	#endif
	#ifdef Gd_ARME_KAMEHA
		UTIL_PrecacheOtherWeapon( "weapon_kameha" );
	#endif
	#ifdef Gd_ARME_KAMIKAZE
		UTIL_PrecacheOtherWeapon( "weapon_kamikaze" );
	#endif
	#ifdef Gd_ARME_MOLOTOV
		UTIL_PrecacheOtherWeapon( "weapon_molotov" );
	#endif
	#ifdef Gd_ARME_MORTIER
		UTIL_PrecacheOtherWeapon( "weapon_mortier" );
	#endif
	#ifdef Gd_ARME_MORVE
		UTIL_PrecacheOtherWeapon( "weapon_morve" );
	#endif
	#ifdef Gd_ARME_MOUTON
		UTIL_PrecacheOtherWeapon( "weapon_mouton" );
	#endif
	#ifdef Gd_ARME_NAPALM
		UTIL_PrecacheOtherWeapon( "weapon_napalm" );
	#endif
	#ifdef Gd_ARME_MINE
		UTIL_PrecacheOtherWeapon( "weapon_mine" );
	#endif
	#ifdef Gd_ARME_MINICANON
		UTIL_PrecacheOtherWeapon( "weapon_minicanon" );
	#endif
	#ifdef Gd_ARME_LASER
		UTIL_PrecacheOtherWeapon( "weapon_laser" );
	#endif
	#ifdef Gd_ARME_PIGEON
		UTIL_PrecacheOtherWeapon( "weapon_pigeon" );
	#endif
	#ifdef Gd_ARME_PHEONIX
		UTIL_PrecacheOtherWeapon( "weapon_pheonix" );
	#endif
	#ifdef Gd_ARME_SAINTE
		UTIL_PrecacheOtherWeapon( "weapon_sainte" );
	#endif
	#ifdef Gd_ARME_SNIPE
		UTIL_PrecacheOtherWeapon( "weapon_snipe" );
	#endif
	#ifdef Gd_ARME_SUPER_MOUTON
		UTIL_PrecacheOtherWeapon( "weapon_super_mouton" );
	#endif
	#ifdef Gd_ARME_TELEPORTEUR
		UTIL_PrecacheOtherWeapon( "weapon_teleporteur" );
	#endif
	#ifdef Gd_ARME_TRIDENT
		UTIL_PrecacheOtherWeapon( "weapon_trident" );
	#endif

	// Groove - 31012002 - Precache caisse
	UTIL_PrecacheOther( "item_baril" );
	UTIL_PrecacheOther( "item_baril_bleu" );
	UTIL_PrecacheOther( "item_baril_rouge" );
	UTIL_PrecacheOther( "item_caisse_arme" );
	UTIL_PrecacheOther( "item_caisse_vie" );
	UTIL_PrecacheOther( "item_caisse_outil" );
	UTIL_PrecacheOther( "item_caisse_mega" );
	
	g_sModelIndexFireball = PRECACHE_MODEL ("sprites/jeu/explosion1.spr");		// explosion 
	g_sModelIndexWExplosion = PRECACHE_MODEL ("sprites/jeu/explosion2.spr");	// Sous l'eau
	g_sModelIndexSmoke = PRECACHE_MODEL ("sprites/jeu/explosion3.spr");			// Fume
	g_sModelIndexBubbles = PRECACHE_MODEL ("sprites/jeu/bulle1.spr");			//bulles dans l'eau

	// Groove - 13022002 - Fumee du au impact.
	PRECACHE_MODEL( Gm_BALLE_FUMEE_1 );
	PRECACHE_MODEL( Gm_BALLE_FUMEE_2 );
	PRECACHE_MODEL( Gm_BALLE_FUMEE_3 );
	PRECACHE_MODEL( Gm_BALLE_FUMEE_4 );

	// used by explosions
	PRECACHE_MODEL ("models/grenade.mdl");
	// Bruit des caisses qui suissent une explosion
	PRECACHE_SOUND ( Gf_SOUND_JEU_CAISSE_CHUTE_01 );

	PRECACHE_SOUND ("weapons/debris1.wav");// explosion aftermaths
	PRECACHE_SOUND ("weapons/debris2.wav");// explosion aftermaths
	PRECACHE_SOUND ("weapons/debris3.wav");// explosion aftermaths

	PRECACHE_SOUND ("weapons/grenade_hit1.wav");//grenade
	PRECACHE_SOUND ("weapons/grenade_hit2.wav");//grenade
	PRECACHE_SOUND ("weapons/grenade_hit3.wav");//grenade
			
	PRECACHE_SOUND ("weapons/bullet_hit1.wav");	// hit by bullet
	PRECACHE_SOUND ("weapons/bullet_hit2.wav");	// hit by bullet
	
	PRECACHE_SOUND( "items/weapondrop1.wav" );// weapon falls to the ground
	PRECACHE_SOUND( "items/9mmclip1.wav" );// weapon falls to the ground

	PRECACHE_SOUND ("weapons/mine/mine1.wav");

	PRECACHE_SOUND( Gf_SOUND_ARMES_PUISSANCE_01 );

	PRECACHE_SOUND( Gm_BALLE_RICOCHET_1 );
	PRECACHE_SOUND( Gm_BALLE_RICOCHET_2 );
	PRECACHE_SOUND( Gm_BALLE_RICOCHET_3 );
	PRECACHE_SOUND( Gm_BALLE_RICOCHET_4 );
	PRECACHE_SOUND( Gm_BALLE_RICOCHET_5 );
	PRECACHE_SOUND( Gm_BALLE_RICOCHET_6 );

	PRECACHE_SOUND( Gm_BALLE_HIT_1 );
	PRECACHE_SOUND( Gm_BALLE_HIT_2 );
	PRECACHE_SOUND( Gm_BALLE_HIT_3 );
	PRECACHE_SOUND( Gm_BALLE_HIT_4 );
	PRECACHE_SOUND( Gm_BALLE_HIT_5 );

	PRECACHE_MODEL( Gm_LASER_FICHIER_SPRITE_DOT_ROUGE );
	PRECACHE_MODEL( Gm_LASER_FICHIER_SPRITE_DOT_BLEU );	

	PRECACHE_MODEL( Gm_SNIPE_FICHIER_MODEL_VUE );
}

void CBasePlayerItem :: SetObjectCollisionBox( void )
{
	pev->absmin = pev->origin + Vector( -24,-24,0 );
	pev->absmax = pev->origin + Vector( 24,24,16 ); 
}

//=========================================================
// Sets up movetype, size, solidtype for a new weapon. 
//=========================================================
void CBasePlayerItem :: FallInit( void )
{
	pev->movetype = MOVETYPE_TOSS;
	pev->solid = SOLID_BBOX;

	UTIL_SetOrigin( pev, pev->origin );
	UTIL_SetSize(pev, Vector( 0,0,0 ), Vector( 0,0,0 ));//pointsize until it lands on the ground.
	
	SetTouch( DefaultTouch );
	SetThink( FallThink );

	pev->nextthink = gpGlobals->time + 0.1;
}

//=========================================================
// FallThink - Items that have just spawned run this think
// to catch them when they hit the ground. Once we're sure
// that the object is grounded, we change its solid type
// to trigger and set it in a large box that helps the
// player get it.
//=========================================================
void CBasePlayerItem::FallThink( void )
{
	pev->nextthink = gpGlobals->time + 0.1;

	if( pev->flags & FL_ONGROUND )
	{
		// clatter if we have an owner (i.e., dropped by someone)
		// don't clatter if the gun is waiting to respawn (if it's waiting, it is invisible!)
		if( !FNullEnt( pev->owner ))
		{
			int pitch = 95 + RANDOM_LONG(0,29);
			EMIT_SOUND_DYN(ENT(pev), CHAN_VOICE, "items/weapondrop1.wav", 1, ATTN_NORM, 0, pitch);	
		}

		// lie flat
		pev->angles.x = 0;
		pev->angles.z = 0;

		Materialize( ); 
	}
}

//=========================================================
// Materialize - make a CBasePlayerItem visible and tangible
//=========================================================
void CBasePlayerItem::Materialize( void )
{
	if( pev->effects & EF_NODRAW )
	{
		pev->effects &= ~EF_NODRAW;
		pev->effects |= EF_MUZZLEFLASH;
	}

	pev->solid = SOLID_TRIGGER;

	UTIL_SetOrigin( pev, pev->origin );// link into world.
	SetTouch (DefaultTouch);
	SetThink (NULL);

}

//=========================================================
// AttemptToMaterialize - the item is trying to rematerialize,
// should it do so now or wait longer?
//=========================================================
void CBasePlayerItem::AttemptToMaterialize( void )
{
	float time = 1.0;

	if( time == 0 )
	{
		Materialize( );
		return;
	}

	pev->nextthink = gpGlobals->time + time;
}

void CBasePlayerItem::DefaultTouch( CBaseEntity *pOther )
{
	// if it's not a player, ignore
	if( !pOther->IsPlayer( ))
		return;

	CBasePlayer *pPlayer = (CBasePlayer *)pOther;

	if( pOther->AddPlayerItem( this ))
		AttachToPlayer( pPlayer );

	SUB_UseTargets( pOther, USE_TOGGLE, 0 ); // UNDONE: when should this happen?
}

int CBasePlayerWeapon::GfDommage( int iGvDommage )
{
	if( m_pPlayer->iGvPlayerOutil & Gm_PLAYER_OUTIL_DOMMAGE_2X )
		return( iGvDommage *= 2 );
	else if( m_pPlayer->iGvPlayerOutil & Gm_PLAYER_OUTIL_DOMMAGE_3X )
		return( iGvDommage *= 3 );
	else 
		return iGvDommage;
}

// Groove - 06032002 - Affichage d'un spot
Vector CBasePlayerWeapon::UpdateSpot( void )
{
	if( !m_pSpot )
		if( FStrEq( m_pPlayer->m_szTeamName,"Wormsblue" ))
			m_pSpot = CLaserSpot::CreateSpotBleu( );
		else
			m_pSpot = CLaserSpot::CreateSpotRouge( );

	UTIL_MakeVectors( m_pPlayer->pev->v_angle );
	Vector vecSrc = m_pPlayer->GetGunPosition( );;
	Vector vecAiming = gpGlobals->v_forward;

	TraceResult tr;
	UTIL_TraceLine( vecSrc, vecSrc + vecAiming * 8192, dont_ignore_monsters, ENT( m_pPlayer->pev ), &tr );
	
	UTIL_SetOrigin( m_pSpot->pev, tr.vecEndPos );
	return m_pSpot->pev->origin;
}

void CBasePlayerWeapon::ItemPostFrame( void )
{
	if(!( m_pPlayer->pev->button &( IN_ATTACK|IN_ATTACK2 )))
		GfAttack0( );
	else if(( m_pPlayer->pev->button & IN_ATTACK )&&( fGvAttack1 <= gpGlobals->time ))
		GfAttack1( );
	else if(( m_pPlayer->pev->button & IN_ATTACK2 )&&( fGvAttack2 <= gpGlobals->time ))
		GfAttack2( );
}

void CBasePlayerItem::DestroyItem( void )
{
	if( m_pPlayer )
		m_pPlayer->RemovePlayerItem( this );

	Kill( );
}


int CBasePlayerItem::AddToPlayer( CBasePlayer *pPlayer )
{
	m_pPlayer = pPlayer;

	return TRUE;
}

void CBasePlayerItem::Drop( void )
{
	SetTouch( NULL );
	SetThink( SUB_Remove );
	pev->nextthink = gpGlobals->time + .1;
}

void CBasePlayerItem::Kill( void )
{
	SetTouch( NULL );
	SetThink( SUB_Remove );
	pev->nextthink = gpGlobals->time + .1;
}

void CBasePlayerItem::Holster( void )
{ 
	m_pPlayer->pev->viewmodel = 0; 
	m_pPlayer->pev->weaponmodel = 0;
}

void CBasePlayerItem::AttachToPlayer ( CBasePlayer *pPlayer )
{
	pev->movetype = MOVETYPE_FOLLOW;
	pev->solid = SOLID_NOT;
	pev->aiment = pPlayer->edict();
	pev->effects = EF_NODRAW; // ??
	pev->modelindex = 0;// server won't send down to clients if modelindex == 0
	pev->model = iStringNull;
	pev->owner = pPlayer->edict();
	pev->nextthink = gpGlobals->time + .1;
	SetTouch( NULL );
}

// CALLED THROUGH the newly-touched weapon's instance. The existing player weapon is pOriginal
int CBasePlayerWeapon::AddDuplicate( CBasePlayerItem *pOriginal )
{
	return ExtractAmmo( (CBasePlayerWeapon *)pOriginal );
}

int CBasePlayerWeapon::AddToPlayer( CBasePlayer *pPlayer )
{
	int bResult = CBasePlayerItem::AddToPlayer( pPlayer );

	//m_pPlayer->iGvArmes |= ( 1<<m_iId );
	// Groove - 08052002 - Gestion des armes de WormsHL
	m_pPlayer->iGvArmes[ItemInfoArray[ m_iId ].iSlot] |= ( 1<<ItemInfoArray[ m_iId ].iPosition );
	
	if ( !iGvAmmoID )
		iGvAmmoID = pPlayer->GetAmmoIndex( ItemInfoArray[m_iId].pszAmmo1 );

	if( bResult )
	{
		ExtractAmmo( this );
		return TRUE;
	}
	return FALSE;
}

int CBasePlayerWeapon::UpdateClientData( CBasePlayer *pPlayer )
{
	BOOL bSend = FALSE;
	int state = 0;
	if( pPlayer->m_pActiveItem == this )
	{
		if ( pPlayer->m_fOnTarget )
			state = WEAPON_IS_ONTARGET;
		else
			state = 1;
	}

	// Forcing send of all data!
	if ( !pPlayer->m_fWeapon )
		bSend = TRUE;
	
	// This is the current or last weapon, so the state will need to be updated
	if( this == pPlayer->m_pActiveItem || this == pPlayer->m_pClientActiveItem )
		if ( pPlayer->m_pActiveItem != pPlayer->m_pClientActiveItem )
			bSend = TRUE;

	// If the ammo, state, or fov has changed, update the weapon
	if ( /*m_iClip != m_iClientClip || */
		 state != m_iClientWeaponState || 
		 pPlayer->m_iFOV != pPlayer->m_iClientFOV )
	{
		bSend = TRUE;
	}

	if( bSend )
	{
		MESSAGE_BEGIN( MSG_ONE, gmsgCurWeapon, NULL, pPlayer->pev );
			WRITE_BYTE( state );
			WRITE_BYTE( m_iId );
		MESSAGE_END();

		m_iClientWeaponState = state;
		pPlayer->m_fWeapon = TRUE;
	}

	if( m_pNext )
		m_pNext->UpdateClientData( pPlayer );

	return 1;
}


void CBasePlayerWeapon::SendWeaponAnim( int iAnim, int skiplocal )
{
	m_pPlayer->pev->weaponanim = iAnim;

#if defined( CLIENT_WEAPONS )
	if ( skiplocal && ENGINE_CANSKIP( m_pPlayer->edict() ) )
		return;
#endif

	MESSAGE_BEGIN( MSG_ONE, SVC_WEAPONANIM, NULL, m_pPlayer->pev );
		WRITE_BYTE( iAnim );						// sequence number
		WRITE_BYTE( pev->body );					// weaponmodel bodygroup.
	MESSAGE_END();
}

BOOL CBasePlayerWeapon::GfAmmoAdd( int iCount, char *szName, int iMaxClip, int iMaxCarry )
{
	int iIdAmmo;

//	m_iClip = -1;
	iIdAmmo = m_pPlayer->GiveAmmo( iCount, szName, iMaxCarry );
	
	// m_pPlayer->m_rgAmmo[iGvAmmoID] = iMaxCarry; // hack for testing

	if( iIdAmmo > 0 )
	{
		iGvAmmoID = iIdAmmo;
		if( m_pPlayer->HasPlayerItem( this ))
			// play the "got ammo" sound only if we gave some ammo to a player that already had this gun.
			// if the player is just getting this gun for the first time, DefaultTouch will play the "picked up gun" sound for us.
			EMIT_SOUND(ENT(pev), CHAN_ITEM, "items/9mmclip1.wav", 1, ATTN_NORM);
	}

	return iIdAmmo > 0 ? TRUE : FALSE;
}

BOOL CBasePlayerWeapon :: CanDeploy( void )
{
	BOOL bHasAmmo = 0;

	if( ItemInfoArray[m_iId].pszAmmo1 )
		bHasAmmo |= ( m_pPlayer->m_rgAmmo[iGvAmmoID] != 0 );
	else
		return TRUE;

	if( !bHasAmmo )
		return FALSE;

	return TRUE;
}

BOOL CBasePlayerWeapon :: DefaultDeploy( char *szViewModel, char *szWeaponModel, int iAnim, char *szAnimExt )
{
	if( !CanDeploy( ))
		return FALSE;

	m_pPlayer->pev->viewmodel = MAKE_STRING( szViewModel );
	m_pPlayer->pev->weaponmodel = MAKE_STRING( szWeaponModel );
	strcpy( m_pPlayer->m_szAnimExtention, szAnimExt );
	SendWeaponAnim( iAnim );

	m_pPlayer->m_flNextAttack = UTIL_WeaponTimeBase( ) + 0.5;
	fGvAttack0 = UTIL_WeaponTimeBase( ) + 1.0;

	return TRUE;
}

void CBasePlayerWeapon::Holster( void )
{ 
	m_pPlayer->pev->viewmodel = 0; 
	m_pPlayer->pev->weaponmodel = 0;
}

//=========================================================
// called by the new item with the existing item as parameter
//
// if we call ExtractAmmo(), it's because the player is picking up this type of weapon for 
// the first time. If it is spawned by the world, m_iDefaultAmmo will have a default ammo amount in it.
// if  this is a weapon dropped by a dying player, has 0 m_iDefaultAmmo, which means only the ammo in 
// the weapon clip comes along. 
//=========================================================
int CBasePlayerWeapon::ExtractAmmo( CBasePlayerWeapon *pWeapon )
{
	int			iReturn;

	if( ItemInfoArray[m_iId].pszAmmo1 )
	{
		// blindly call with m_iDefaultAmmo. It's either going to be a value or zero. If it is zero,
		// we only get the ammo in the weapon's clip, which is what we want. 
		iReturn = pWeapon->GfAmmoAdd( iGvAmmoDefault, (char *)ItemInfoArray[m_iId].pszAmmo1, -1, ItemInfoArray[m_iId].iMaxAmmo1 );
		iGvAmmoDefault = 0;
	}

	return iReturn;
}

// Groove - Intersection - 07112001
void CBasePlayerWeapon::FindHullIntersection( const Vector &vecSrc, TraceResult &tr, float *mins, float *maxs, edict_t *pEntity )
{
	int			i, j, k;
	float		distance;
	float		*minmaxs[2] = {mins, maxs};
	TraceResult tmpTrace;
	Vector		vecHullEnd = tr.vecEndPos;
	Vector		vecEnd;

	distance = 1e6f;

	vecHullEnd = vecSrc + ((vecHullEnd - vecSrc)*2);
	UTIL_TraceLine( vecSrc, vecHullEnd, dont_ignore_monsters, pEntity, &tmpTrace );
	if( tmpTrace.flFraction < 1.0 )
	{
		tr = tmpTrace;
		return;
	}

	for ( i = 0; i < 2; i++ )
		for ( j = 0; j < 2; j++ )
			for ( k = 0; k < 2; k++ )
			{
				vecEnd.x = vecHullEnd.x + minmaxs[i][0];
				vecEnd.y = vecHullEnd.y + minmaxs[j][1];
				vecEnd.z = vecHullEnd.z + minmaxs[k][2];

				UTIL_TraceLine( vecSrc, vecEnd, dont_ignore_monsters, pEntity, &tmpTrace );
				if ( tmpTrace.flFraction < 1.0 )
				{
					float thisDistance = (tmpTrace.vecEndPos - vecSrc).Length();
					if ( thisDistance < distance )
					{
						tr = tmpTrace;
						distance = thisDistance;
					}
				}
			}
}


void CBasePlayerWeapon::GfMsgStupide( CBasePlayer *m_pPlayer )
{
	m_pPlayer->sGvProfil.iGvStupide++;
		
	short iGvMsg;
	if( m_pPlayer && ( FStrEq( m_pPlayer->m_szTeamName,"Wormsred" )))
		iGvMsg = RANDOM_LONG( Gm_MSG_ROUGE_STUPIDE_01, Gm_MSG_ROUGE_STUPIDE_04 );
	else
		iGvMsg = RANDOM_LONG( Gm_MSG_BLEU_STUPIDE_01, Gm_MSG_BLEU_STUPIDE_04 );

	MESSAGE_BEGIN( MSG_ONE, gmsgBarreMsg, NULL, m_pPlayer->pev );
		WRITE_BYTE( iGvMsg );
	MESSAGE_END( );
}

//Groove, Son pour la puissance progressive ,07102001
void CBasePlayerWeapon::GfSonPuissanceOn( void )
{
	EMIT_SOUND( ENT( m_pPlayer->pev ), CHAN_WEAPON, Gf_SOUND_ARMES_PUISSANCE_01, 0.9, ATTN_NORM );
}

void CBasePlayerWeapon::GfSonAttack( void )
{
	switch( RANDOM_LONG( 1,8 ))
	{
		case 1 : EMIT_SOUND( ENT( m_pPlayer->pev ), CHAN_VOICE, Gf_SOUND_AMBIANCE_ATTACK_01, 1, ATTN_NORM ); break;
		case 2 : EMIT_SOUND( ENT( m_pPlayer->pev ), CHAN_VOICE, Gf_SOUND_AMBIANCE_ATTACK_02, 1, ATTN_NORM ); break;
		case 3 : EMIT_SOUND( ENT( m_pPlayer->pev ), CHAN_VOICE, Gf_SOUND_AMBIANCE_ATTACK_03, 1, ATTN_NORM ); break;
		case 4 : EMIT_SOUND( ENT( m_pPlayer->pev ), CHAN_VOICE, Gf_SOUND_AMBIANCE_ATTACK_04, 1, ATTN_NORM ); break;
		case 5 : EMIT_SOUND( ENT( m_pPlayer->pev ), CHAN_VOICE, Gf_SOUND_AMBIANCE_ATTACK_05, 1, ATTN_NORM ); break;
		case 6 : EMIT_SOUND( ENT( m_pPlayer->pev ), CHAN_VOICE, Gf_SOUND_AMBIANCE_ATTACK_06, 1, ATTN_NORM ); break;
		case 7 : EMIT_SOUND( ENT( m_pPlayer->pev ), CHAN_VOICE, Gf_SOUND_AMBIANCE_ATTACK_07, 1, ATTN_NORM ); break;
		case 8 : EMIT_SOUND( ENT( m_pPlayer->pev ), CHAN_VOICE, Gf_SOUND_AMBIANCE_ATTACK_08, 1, ATTN_NORM ); break;
		case 9 : EMIT_SOUND( ENT( m_pPlayer->pev ), CHAN_VOICE, Gf_SOUND_AMBIANCE_ATTACK_09, 1, ATTN_NORM ); break;
	}
}

void CBasePlayerWeapon::GfPuissanceAffichage( short iGvPuis, short iGvCouleur )
{
	if( iGvPuis == WEAPON_PUISS_PROG_OFF )
		STOP_SOUND( ENT( m_pPlayer->pev ), CHAN_WEAPON, Gf_SOUND_ARMES_PUISSANCE_01 );

	MESSAGE_BEGIN( MSG_ONE, gmsgPuissance, NULL, m_pPlayer->pev );
		WRITE_BYTE( iGvPuis );
		WRITE_BYTE( iGvCouleur );
	MESSAGE_END( );
}

bool CBasePlayerWeapon::GfPuissanceProgressive( float fGvTempsPuis )
{
	if(( gpGlobals->time  - fGvTempsPuis ) >= Gm_ARME_TEMPS_TIRE_MAX )
	{
		if( m_pPlayer && ( FStrEq( m_pPlayer->m_szTeamName,"Wormsred" )))
			GfPuissanceAffichage( 100, Gm_WORM_ROUGE );		
        if( m_pPlayer && ( FStrEq( m_pPlayer->m_szTeamName,"Wormsblue" )))
			GfPuissanceAffichage( 100, Gm_WORM_BLEU );

		return true;
	}
	else
	{
		if( m_pPlayer && ( FStrEq( m_pPlayer->m_szTeamName,"Wormsred" )))
			GfPuissanceAffichage(( int(( gpGlobals->time - fGvTempsPuis ) * ( 100 / Gm_ARME_TEMPS_TIRE_MAX ))),Gm_WORM_ROUGE );
        if( m_pPlayer && ( FStrEq( m_pPlayer->m_szTeamName,"Wormsblue" )))
			GfPuissanceAffichage(( int(( gpGlobals->time - fGvTempsPuis ) * ( 100 / Gm_ARME_TEMPS_TIRE_MAX ))),Gm_WORM_BLEU );

		return false;
	}
}

void CBasePlayerWeapon::GfFinDeTour( float *fGvTempsFDT, float fGvTempsDeplacement )
{
	if( g_pGameRules->iGfGamerulesActif( ) == Gm_GAMERULES_ACTION )
		return;
	
	if((( gpGlobals->time - *fGvTempsFDT ) >= fGvTempsDeplacement ) && *fGvTempsFDT )
	{
		g_pGameRules->GfFinDeTourAutoTPT( m_pPlayer ); 
		m_pPlayer->GvPlayerTPT.bMove = false;
		m_pPlayer->GvPlayerTPT.bTire = false;
		m_pPlayer->pev->maxspeed = 1;
		*fGvTempsFDT = 0;
	}
}

void CBasePlayerWeapon::GfZoomDynamic( char *pcGvViewModel, char *pcGvWeaponModel, int iGvZoomMini, int iGvZoomMaxi )
{
	if( bGvZoomRetireForce )
	{
		/*m_pPlayer->pev->fov = */m_pPlayer->m_iFOV = iGvZoom = 0;
		m_pPlayer->pev->viewmodel = MAKE_STRING( pcGvViewModel );
		m_pPlayer->pev->weaponmodel = MAKE_STRING( pcGvWeaponModel );

		if( g_pGameRules->GvMortSubite )
			UTIL_ScreenFade( m_pPlayer, Vector( 255, 0, 0 ), 1, 0.5, 64, FFADE_OUT | FFADE_STAYOUT );
		else
			UTIL_ScreenFade( m_pPlayer, Vector( 255,255,255 ), 0.5, 0.25, 128, FFADE_IN );

		fGvAttack1 = fGvAttack2 = gpGlobals->time + 1.5;
		return;
	}

	if( !iGvZoom )
	{
		m_pPlayer->pev->viewmodel = 0;
		m_pPlayer->pev->weaponmodel = 0;

		if( g_pGameRules->GvMortSubite )
			UTIL_ScreenFade( m_pPlayer, Vector( 255, 0, 0 ), 1, 0.5, 64, FFADE_OUT | FFADE_STAYOUT );
		else
			UTIL_ScreenFade( m_pPlayer, Vector( 1,1,1 ), 2.0, 0.25, 255, FFADE_IN );
		
		/*m_pPlayer->pev->fov = */m_pPlayer->m_iFOV = iGvZoom = ( iGvZoomMini + iGvZoomMaxi ) / 2;
		bGvZoomSensMoins = false;
		fGvAttack1 = fGvAttack2 = gpGlobals->time + 2.0;
		return;
	}
	else if( bGvZoomSensChange )
	{
		bGvZoomSensChange = false;
		if( bGvZoomSensMoins )
			bGvZoomSensMoins = false;
		else
			bGvZoomSensMoins = true;
	}

	if( bGvZoomSensMoins )
	{
		iGvZoom--;
		if( iGvZoom < iGvZoomMaxi )
		{
			iGvZoom = iGvZoomMaxi;
			bGvZoomSensMoins = false;
		}
	}
	else
	{
		iGvZoom++;
		if( iGvZoom >= iGvZoomMini )
		{
			/*m_pPlayer->pev->fov = */m_pPlayer->m_iFOV = iGvZoom = 0;
			m_pPlayer->pev->viewmodel = MAKE_STRING( pcGvViewModel );
			m_pPlayer->pev->weaponmodel = MAKE_STRING( pcGvWeaponModel );

			if( g_pGameRules->GvMortSubite )
				UTIL_ScreenFade( m_pPlayer, Vector( 255, 0, 0 ), 1, 0.5, 64, FFADE_OUT | FFADE_STAYOUT );
			else
				UTIL_ScreenFade( m_pPlayer, Vector( 255,255,255 ), 0.5, 0.25, 128, FFADE_IN );

			fGvAttack1 = fGvAttack2 = gpGlobals->time + 1.5;
			return;
		}
	}

	/*m_pPlayer->pev->fov = */m_pPlayer->m_iFOV = iGvZoom;

	if(( iGvZoom == iGvZoomMaxi )||( iGvZoom == iGvZoomMini ))
		fGvAttack2 = gpGlobals->time + 2.0;
	else
		fGvAttack2 = gpGlobals->time + 0.05;

	fGvAttack1 = gpGlobals->time + 0.5;
}
