/*******************************\
*	WormsHL						*
*		www.wormshl.fr.st		*
*								*
*	Groove						*
*		groove@wormshl.net		*
*								*
*	generic_arme.h				*
*		22032002				*
\*******************************/

#ifndef GENERIC_ARME_H
#define GENERIC_ARME_H

#include "generic_spot.h"
#include "generic_grenade.h"

// Groove - 09112001 - pour definir la trajection speudo alatoire des grenades type cluster
#define Gm_RANDOM_VECTOR( GvPreci, GvHauteurMini, GvHauteurMaxi ) ( Vector( RANDOM_LONG( -GvPreci, GvPreci ), RANDOM_LONG( -GvPreci, GvPreci ), RANDOM_LONG( GvHauteurMini, GvHauteurMaxi )) * RANDOM_LONG( 10, 20 ))

class CBasePlayer;

#define ITEM_ANTIDOTE		2

#define WEAPON_ALLWEAPONS			(~(1<<MAX_WEAPONS))
#define WEAPON_PUISS_PROG_OFF		110

#define ITEM_FLAG_SELECTONEMPTY		1
#define ITEM_FLAG_NOAUTORELOAD		2
#define ITEM_FLAG_NOAUTOSWITCHEMPTY	4
#define ITEM_FLAG_LIMITINWORLD		8
#define ITEM_FLAG_EXHAUSTIBLE		16 // A player can totally exhaust their ammo supply and lose this weapon

#define WEAPON_IS_ONTARGET 0x40

typedef struct
{
	int		iSlot;
	int		iPosition;
	const char	*pszAmmo1;	// ammo 1 type
	int		iMaxAmmo1;		// max ammo 1
	const char	*pszName;
	int		iId;
	int		iFlags;
	int		iWeight;// this value used to determine this weapon's importance in autoselection.
} ItemInfo;

typedef struct
{
	const char *pszName;
	int iId;
} AmmoInfo;

// Items that the player has in their inventory that they can use
class CBasePlayerItem : public CBaseAnimating
{
public:
	virtual void SetObjectCollisionBox( void );

	virtual int AddToPlayer( CBasePlayer *pPlayer );	// return TRUE if the item you want the item added to the player inventory
	virtual int AddDuplicate( CBasePlayerItem *pItem ) { return FALSE; }	// return TRUE if you want your duplicate removed from world
	void EXPORT DestroyItem( void );
	void EXPORT DefaultTouch( CBaseEntity *pOther );	// default weapon touch
	void EXPORT FallThink ( void );// when an item is first spawned, this think is run to determine when the object has hit the ground.
	void EXPORT Materialize( void );// make a weapon visible and tangible
	void EXPORT AttemptToMaterialize( void );  // the weapon desires to become visible and tangible, if the game rules allow for it

	void FallInit( void );
	virtual int GetItemInfo(ItemInfo *p) { return 0; };	// returns 0 if struct not filled out
	virtual BOOL CanDeploy( void ){ return TRUE; };
	virtual BOOL Deploy( ){ return TRUE; };

	virtual BOOL CanHolster( void ) { return TRUE; };// can this weapon be put away right now?
	virtual void Holster( void );
	virtual void UpdateItemInfo( void ) { return; };

	virtual void ItemPreFrame( void )	{ return; }		// called each frame by the player PreThink
	virtual void ItemPostFrame( void ) { return; }		// called each frame by the player PostThink

	virtual void Drop( void );
	virtual void Kill( void );
	virtual void AttachToPlayer ( CBasePlayer *pPlayer );
	virtual int UpdateClientData( CBasePlayer *pPlayer ) { return 0; }

	virtual CBasePlayerItem *GetWeaponPtr( void ) { return NULL; };

	static ItemInfo ItemInfoArray[ MAX_WEAPONS ];
	static AmmoInfo AmmoInfoArray[ MAX_AMMO_SLOTS ];

	CBasePlayer	*m_pPlayer;
	CBasePlayerItem *m_pNext;
	int		m_iId;												// WEAPON_???

	int			iItemSlot( void ){ return ItemInfoArray[m_iId].iSlot + 1; }			// return 0 to MAX_ITEMS_SLOTS, used in hud
};

class CBasePlayerWeapon : public CBasePlayerItem
{
public:
	// generic weapon versions of CBasePlayerItem calls
	virtual int AddToPlayer( CBasePlayer *pPlayer );
	virtual int AddDuplicate( CBasePlayerItem *pItem );

	virtual int ExtractAmmo( CBasePlayerWeapon *pWeapon ); //{ return TRUE; };			// Return TRUE if you can add ammo to yourself when picked up

	// generic "shared" ammo handlers
	BOOL GfAmmoAdd( int iCount, char *szName, int iMaxClip, int iMaxCarry );

	virtual void UpdateItemInfo( void ) {};	// updates HUD state

	virtual void SendWeaponAnim( int iAnim, int skiplocal = 0 );  // skiplocal is 1 if client is predicting weapon animations

	virtual BOOL CanDeploy( void );
	BOOL DefaultDeploy( char *szViewModel, char *szWeaponModel, int iAnim, char *szAnimExt );

	virtual void ItemPostFrame( void );	// called each frame by the player PostThink
	virtual void GfAttack1( void ) { return; }				// do "+ATTACK"
	virtual void GfAttack2( void ) { return; }				// do "+ATTACK2"
	virtual void GfAttack0( void ) { return; }				// called when no buttons pressed
	virtual int UpdateClientData( CBasePlayer *pPlayer );	// sends hud info to client dll, if things have changed
	virtual void Holster( void );
	virtual BOOL UseDecrement( void ) { return FALSE; };
	// Groove - 06032002 - Affichage d'un spot
	virtual Vector UpdateSpot( void );
	CLaserSpot *m_pSpot;
	// Groove - 14032002 - Calcule d'eventuel modification des dommages
	int GfDommage( int iGvDommage );

	virtual CBasePlayerItem *GetWeaponPtr( void ) { return (CBasePlayerItem *)this; };

	float fGvAttack0;		// Groove - 22052002 - Idle
	float fGvAttack1;		// Groove - 22052002 - Attack 1
	float fGvAttack2;		// Groove - 22052002 - Attack 2
	int iGvAmmoID;			// Groove - 22052002 - Index des munitions
	int iGvAmmoDefault;		// Groove - 22052002 - Nombre de munitions dans une arme par default

	int		m_iClientWeaponState;								// the last version of the weapon state sent to hud dll (is current weapon, is on target)

	// Groove - 30102001 - fonction generic pour les armes.
	void FindHullIntersection( const Vector &vecSrc, TraceResult &tr, float *mins, float *maxs, edict_t *pEntity );
	void GfFinDeTour( float *fGvTempsFDT, float fGvTempsDeplacement );
	bool GfPuissanceProgressive( float fGvTempsPuis );
	void GfPuissanceAffichage( short iGvPuis, short iGvCouleur );

	void GfSonPuissanceOn( void );
	void GfSonAttack( void );
	// Groove - 12122001 - Pour envoyer un msg type "stupide" quand le joueur fait un caouette
	void GfMsgStupide( CBasePlayer *m_pPlayer );

	// Groove - 08032002 - Compte les rocket active.
	int GvRocketActive_i;
	Vector GvPosition_c;

	// Groove - 21052002 - Zoom
	void GfZoomDynamic( char *pcGvViewModel, char *pcGvWeaponModel, int iGvZoomMini, int iGvZoomMaxi );
	int iGvZoom;
	bool bGvZoomRetireForce;
	bool bGvZoomSensMoins;
	bool bGvZoomSensChange;
};

extern DLL_GLOBAL	short	g_sModelIndexLaser;// holds the index for the laser beam
extern DLL_GLOBAL	short	g_sModelIndexFireball;// holds the index for the fireball
extern DLL_GLOBAL	short	g_sModelIndexSmoke;// holds the index for the smoke cloud
extern DLL_GLOBAL	short	g_sModelIndexWExplosion;// holds the index for the underwater explosion
extern DLL_GLOBAL	short	g_sModelIndexBubbles;// holds the index for the bubbles model

extern void ClearMultiDamage( void );
extern void ApplyMultiDamage( entvars_t* pevInflictor, entvars_t* pevAttacker );
extern void AddMultiDamage( entvars_t *pevInflictor, CBaseEntity *pEntity, float flDamage, int bitsDamageType);

extern void RadiusDamage( Vector vecSrc, entvars_t *pevInflictor, entvars_t *pevAttacker, float flDamage, float flRadius, int iClassIgnore, int bitsDamageType );

typedef struct 
{
	CBaseEntity		*pEntity;
	float			amount;
	int				type;
} MULTIDAMAGE;

extern MULTIDAMAGE gMultiDamage;

#define	BRIGHT_GUN_FLASH		512
#define NORMAL_GUN_FLASH		256
#define	DIM_GUN_FLASH			128

#endif // GENERIC_ARME_H
